# pygpgme - a Python wrapper for the gpgme library
# Copyright (C) 2006  James Henstridge
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import unittest

import gpgme
from tests.util import GpgHomeTestCase

class KeylistTestCase(GpgHomeTestCase):

    import_keys = ['key1.pub', 'key2.pub', 'revoked.pub', 'signonly.pub',
                   'key1.sec']

    def test_listall(self) -> None:
        ctx = gpgme.Context()
        keyids = set(key.subkeys[0].keyid for key in ctx.keylist())
        self.assertTrue(keyids, set(['46BB55F0885C65A4',
                                     '2CF46B7FC97E6B0F',
                                     'F540A569CB935A42',
                                     '2EF658C987754368']))

    def test_list_by_email(self) -> None:
        ctx = gpgme.Context()
        keyids = set(key.subkeys[0].keyid
                     for key in ctx.keylist('key1@example.org'))
        self.assertTrue(keyids, set(['46BB55F0885C65A4']))
        keyids = set(key.subkeys[0].keyid
                     for key in ctx.keylist(['key1@example.org',
                                             'signonly@example.com']))
        self.assertTrue(keyids, set(['46BB55F0885C65A4', 'F540A569CB935A42']))

    def test_list_by_name(self) -> None:
        ctx = gpgme.Context()
        keyids = set(key.subkeys[0].keyid
                     for key in ctx.keylist('Key 1'))
        self.assertTrue(keyids, set(['46BB55F0885C65A4']))

    def test_list_by_email_substring(self) -> None:
        ctx = gpgme.Context()
        keyids = set(key.subkeys[0].keyid
                     for key in ctx.keylist('@example.org'))
        self.assertTrue(keyids, set(['46BB55F0885C65A4',
                                     '2CF46B7FC97E6B0F',
                                     'F540A569CB935A42',
                                     '2EF658C987754368']))

    def test_list_secret(self) -> None:
        ctx = gpgme.Context()
        keyids = set(key.subkeys[0].keyid
                     for key in ctx.keylist(None, True))
        self.assertTrue(keyids, set(['46BB55F0885C65A4']))



# pygpgme - a Python wrapper for the gpgme library
# Copyright (C) 2006  James Henstridge
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import unittest

import gpgme
from tests.util import GpgHomeTestCase

class KeyTestCase(GpgHomeTestCase):

    import_keys = ['key1.pub', 'key2.pub', 'revoked.pub', 'signonly.pub']

    def test_key1(self) -> None:
        ctx = gpgme.Context()
        key = ctx.get_key('E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        self.assertEqual(key.revoked, False)
        self.assertEqual(key.expired, False)
        self.assertEqual(key.invalid, False)
        self.assertEqual(key.can_encrypt, True)
        self.assertEqual(key.can_sign, True)
        self.assertEqual(key.can_certify, True)
        self.assertEqual(key.secret, False)
        self.assertEqual(key.can_authenticate, False)
        self.assertEqual(key.protocol, gpgme.Protocol.OpenPGP)
        self.assertEqual(len(key.subkeys), 2)
        self.assertEqual(len(key.uids), 1)

        self.assertEqual(key.subkeys[0].revoked, False)
        self.assertEqual(key.subkeys[0].expired, False)
        self.assertEqual(key.subkeys[0].disabled, False)
        self.assertEqual(key.subkeys[0].invalid, False)
        self.assertEqual(key.subkeys[0].can_encrypt, False)
        self.assertEqual(key.subkeys[0].can_sign, True)
        self.assertEqual(key.subkeys[0].can_certify, True)
        self.assertEqual(key.subkeys[0].secret, False)
        self.assertEqual(key.subkeys[0].can_authenticate, False)
        self.assertEqual(key.subkeys[0].pubkey_algo, gpgme.PubkeyAlgo.DSA)
        self.assertEqual(key.subkeys[0].length, 1024)
        self.assertEqual(key.subkeys[0].keyid, '46BB55F0885C65A4')
        self.assertEqual(key.subkeys[0].fpr,
                         'E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        self.assertEqual(key.subkeys[0].timestamp, 1137568227)
        self.assertEqual(key.subkeys[0].expires, 0)

        self.assertEqual(key.subkeys[1].revoked, False)
        self.assertEqual(key.subkeys[1].expired, False)
        self.assertEqual(key.subkeys[1].disabled, False)
        self.assertEqual(key.subkeys[1].invalid, False)
        self.assertEqual(key.subkeys[1].can_encrypt, True)
        self.assertEqual(key.subkeys[1].can_sign, False)
        self.assertEqual(key.subkeys[1].can_certify, False)
        self.assertEqual(key.subkeys[1].secret, False)
        self.assertEqual(key.subkeys[1].can_authenticate, False)
        self.assertEqual(key.subkeys[1].pubkey_algo, gpgme.PubkeyAlgo.ELG_E)
        self.assertEqual(key.subkeys[1].length, 2048)
        self.assertEqual(key.subkeys[1].keyid, '659A6AC69BC3B085')
        # Some versions of libgpgme fill this one in and others don't
        #self.assertEqual(key.subkeys[1].fpr, None)
        self.assertEqual(key.subkeys[1].timestamp, 1137568234)
        self.assertEqual(key.subkeys[1].expires, 0)

        self.assertEqual(key.uids[0].revoked, False)
        self.assertEqual(key.uids[0].invalid, False)
        self.assertEqual(key.uids[0].validity, 0)
        self.assertEqual(key.uids[0].uid, 'Key 1 <key1@example.org>')
        self.assertEqual(key.uids[0].name, 'Key 1')
        self.assertEqual(key.uids[0].email, 'key1@example.org')
        self.assertEqual(key.uids[0].comment, '')

    def test_key2(self) -> None:
        ctx = gpgme.Context()
        key = ctx.get_key('93C2240D6B8AA10AB28F701D2CF46B7FC97E6B0F')
        self.assertEqual(key.revoked, False)
        self.assertEqual(key.expired, False)
        self.assertEqual(key.invalid, False)
        self.assertEqual(key.can_encrypt, True)
        self.assertEqual(key.can_sign, True)
        self.assertEqual(key.can_certify, True)
        self.assertEqual(key.secret, False)
        self.assertEqual(key.can_authenticate, False)
        self.assertEqual(key.protocol, gpgme.Protocol.OpenPGP)
        self.assertEqual(len(key.subkeys), 2)
        self.assertEqual(len(key.uids), 1)

        self.assertEqual(key.subkeys[0].revoked, False)
        self.assertEqual(key.subkeys[0].expired, False)
        self.assertEqual(key.subkeys[0].disabled, False)
        self.assertEqual(key.subkeys[0].invalid, False)
        self.assertEqual(key.subkeys[0].can_encrypt, False)
        self.assertEqual(key.subkeys[0].can_sign, True)
        self.assertEqual(key.subkeys[0].can_certify, True)
        self.assertEqual(key.subkeys[0].secret, False)
        self.assertEqual(key.subkeys[0].can_authenticate, False)
        self.assertEqual(key.subkeys[0].pubkey_algo, gpgme.PubkeyAlgo.RSA)
        self.assertEqual(key.subkeys[0].length, 4096)
        self.assertEqual(key.subkeys[0].keyid, '2CF46B7FC97E6B0F')
        self.assertEqual(key.subkeys[0].fpr,
                         '93C2240D6B8AA10AB28F701D2CF46B7FC97E6B0F')
        self.assertEqual(key.subkeys[0].timestamp, 1137568343)
        self.assertEqual(key.subkeys[0].expires, 0)

        self.assertEqual(key.subkeys[1].revoked, False)
        self.assertEqual(key.subkeys[1].expired, False)
        self.assertEqual(key.subkeys[1].disabled, False)
        self.assertEqual(key.subkeys[1].invalid, False)
        self.assertEqual(key.subkeys[1].can_encrypt, True)
        self.assertEqual(key.subkeys[1].can_sign, False)
        self.assertEqual(key.subkeys[1].can_certify, False)
        self.assertEqual(key.subkeys[1].secret, False)
        self.assertEqual(key.subkeys[1].can_authenticate, False)
        self.assertEqual(key.subkeys[1].pubkey_algo, gpgme.PubkeyAlgo.RSA)
        self.assertEqual(key.subkeys[1].length, 4096)
        self.assertEqual(key.subkeys[1].keyid, 'A95221D00DCBDD64')
        # Some versions of libgpgme fill this one in and others don't
        #self.assertEqual(key.subkeys[1].fpr, None)
        self.assertEqual(key.subkeys[1].timestamp, 1137568395)
        self.assertEqual(key.subkeys[1].expires, 0)

        self.assertEqual(key.uids[0].revoked, False)
        self.assertEqual(key.uids[0].invalid, False)
        self.assertEqual(key.uids[0].validity, 0)
        self.assertEqual(key.uids[0].uid, 'Key 2 <key2@example.org>')
        self.assertEqual(key.uids[0].name, 'Key 2')
        self.assertEqual(key.uids[0].email, 'key2@example.org')
        self.assertEqual(key.uids[0].comment, '')

    def test_revoked(self) -> None:
        ctx = gpgme.Context()
        key = ctx.get_key('B6525A39EB81F88B4D2CFB3E2EF658C987754368')
        self.assertEqual(key.revoked, True)
        self.assertEqual(key.expired, False)
        self.assertEqual(key.invalid, False)
        self.assertEqual(key.can_encrypt, False)
        self.assertEqual(key.can_sign, True)
        self.assertEqual(key.can_certify, True)
        self.assertEqual(key.secret, False)
        self.assertEqual(key.can_authenticate, False)
        self.assertEqual(key.protocol, gpgme.Protocol.OpenPGP)
        self.assertEqual(len(key.subkeys), 2)
        self.assertEqual(len(key.uids), 1)

        self.assertEqual(key.subkeys[0].revoked, True)
        self.assertEqual(key.subkeys[0].expired, False)
        self.assertEqual(key.subkeys[0].disabled, False)
        self.assertEqual(key.subkeys[0].invalid, False)
        self.assertEqual(key.subkeys[0].can_encrypt, False)
        self.assertEqual(key.subkeys[0].can_sign, True)
        self.assertEqual(key.subkeys[0].can_certify, True)
        self.assertEqual(key.subkeys[0].secret, False)
        self.assertEqual(key.subkeys[0].can_authenticate, False)
        self.assertEqual(key.subkeys[0].pubkey_algo, gpgme.PubkeyAlgo.DSA)
        self.assertEqual(key.subkeys[0].length, 1024)
        self.assertEqual(key.subkeys[0].keyid, '2EF658C987754368')
        self.assertEqual(key.subkeys[0].fpr,
                         'B6525A39EB81F88B4D2CFB3E2EF658C987754368')
        self.assertEqual(key.subkeys[0].timestamp, 1137569043)
        self.assertEqual(key.subkeys[0].expires, 0)

        self.assertEqual(key.subkeys[1].revoked, True)
        self.assertEqual(key.subkeys[1].expired, False)
        self.assertEqual(key.subkeys[1].disabled, False)
        self.assertEqual(key.subkeys[1].invalid, False)
        self.assertEqual(key.subkeys[1].can_encrypt, True)
        self.assertEqual(key.subkeys[1].can_sign, False)
        self.assertEqual(key.subkeys[1].can_certify, False)
        self.assertEqual(key.subkeys[1].secret, False)
        self.assertEqual(key.subkeys[1].can_authenticate, False)
        self.assertEqual(key.subkeys[1].pubkey_algo, gpgme.PubkeyAlgo.ELG_E)
        self.assertEqual(key.subkeys[1].length, 1024)
        self.assertEqual(key.subkeys[1].keyid, 'E50B59CF50CE4D54')
        # Some versions of libgpgme fill this one in and others don't
        #self.assertEqual(key.subkeys[1].fpr, None)
        self.assertEqual(key.subkeys[1].timestamp, 1137569047)
        self.assertEqual(key.subkeys[1].expires, 0)

        self.assertEqual(key.uids[0].revoked, True)
        self.assertEqual(key.uids[0].invalid, False)
        self.assertEqual(key.uids[0].validity, 0)
        self.assertEqual(key.uids[0].uid, 'Revoked <revoked@example.org>')
        self.assertEqual(key.uids[0].name, 'Revoked')
        self.assertEqual(key.uids[0].email, 'revoked@example.org')
        self.assertEqual(key.uids[0].comment, '')

    def test_signonly(self) -> None:
        ctx = gpgme.Context()
        key = ctx.get_key('15E7CE9BF1771A4ABC550B31F540A569CB935A42')
        self.assertEqual(key.revoked, False)
        self.assertEqual(key.expired, False)
        self.assertEqual(key.invalid, False)
        self.assertEqual(key.can_encrypt, False)
        self.assertEqual(key.can_sign, True)
        self.assertEqual(key.can_certify, True)
        self.assertEqual(key.secret, False)
        self.assertEqual(key.can_authenticate, False)
        self.assertEqual(key.protocol, gpgme.Protocol.OpenPGP)
        self.assertEqual(len(key.subkeys), 1)
        self.assertEqual(len(key.uids), 2)

        self.assertEqual(key.subkeys[0].revoked, False)
        self.assertEqual(key.subkeys[0].expired, False)
        self.assertEqual(key.subkeys[0].disabled, False)
        self.assertEqual(key.subkeys[0].invalid, False)
        self.assertEqual(key.subkeys[0].can_encrypt, False)
        self.assertEqual(key.subkeys[0].can_sign, True)
        self.assertEqual(key.subkeys[0].can_certify, True)
        self.assertEqual(key.subkeys[0].secret, False)
        self.assertEqual(key.subkeys[0].can_authenticate, False)
        self.assertEqual(key.subkeys[0].pubkey_algo, gpgme.PubkeyAlgo.RSA)
        self.assertEqual(key.subkeys[0].length, 4096)
        self.assertEqual(key.subkeys[0].keyid, 'F540A569CB935A42')
        self.assertEqual(key.subkeys[0].fpr,
                         '15E7CE9BF1771A4ABC550B31F540A569CB935A42')
        self.assertEqual(key.subkeys[0].timestamp, 1137568835)
        self.assertEqual(key.subkeys[0].expires, 0)

        self.assertEqual(key.uids[0].revoked, False)
        self.assertEqual(key.uids[0].invalid, False)
        self.assertEqual(key.uids[0].validity, 0)
        self.assertEqual(key.uids[0].uid, 'Sign Only <signonly@example.org>')
        self.assertEqual(key.uids[0].name, 'Sign Only')
        self.assertEqual(key.uids[0].email, 'signonly@example.org')
        self.assertEqual(key.uids[0].comment, '')

        self.assertEqual(key.uids[1].revoked, False)
        self.assertEqual(key.uids[1].invalid, False)
        self.assertEqual(key.uids[1].validity, 0)
        self.assertEqual(key.uids[1].uid,
                         'Sign Only (work address) <signonly@example.com>')
        self.assertEqual(key.uids[1].name, 'Sign Only')
        self.assertEqual(key.uids[1].email, 'signonly@example.com')
        self.assertEqual(key.uids[1].comment, 'work address')



# pygpgme - a Python wrapper for the gpgme library
# Copyright (C) 2006  James Henstridge
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from io import BytesIO
import unittest

import gpgme
from tests.util import GpgHomeTestCase


# See /usr/share/doc/gnupg/DETAILS.gz

# XXX we are using a passwordless key because the passphrase_cb
# backend seems to be currently broken.

signing_only_param = """
<GnupgKeyParms format="internal">
  Key-Type: RSA
  Key-Usage: sign
  Key-Length: 1024
  Name-Real: Testing
  Name-Comment: comment
  Name-Email: someone@example.com
  Expire-Date: 0
</GnupgKeyParms>
"""


class GenerateKeyTestCase(GpgHomeTestCase):

    def assertCanSign(self, key: gpgme.Key) -> None:
        """Check that the given key can be used to create signatures."""
        ctx = gpgme.Context()
        ctx.signers = [key]

        plaintext = BytesIO(b'Hello World\n')
        signature = BytesIO()

        ctx.armor = True
        new_sigs = ctx.sign(plaintext, signature, gpgme.SigMode.DETACH)

        signature.seek(0)
        plaintext.seek(0)

        sigs = ctx.verify(signature, plaintext, None)
        self.assertEqual(len(sigs), 1)
        self.assertEqual(sigs[0].fpr, key.subkeys[0].fpr)

    def _test_generate_signing_only_keys(self) -> None:
        ctx = gpgme.Context()
        result = ctx.genkey(signing_only_param)

        self.assertEqual(result.primary, True)
        self.assertEqual(result.sub, False)
        self.assertEqual(len(result.fpr), 40)

        # The generated key is part of the current keyring.
        key = ctx.get_key(result.fpr, True)
        self.assertEqual(key.revoked, False)
        self.assertEqual(key.expired, False)
        self.assertEqual(key.secret, True)
        self.assertEqual(key.protocol, gpgme.Protocol.OpenPGP)

        # Single signing-only RSA key.
        self.assertEqual(len(key.subkeys), 1)
        subkey = key.subkeys[0]
        self.assertEqual(subkey.secret, True)
        self.assertEqual(subkey.pubkey_algo, gpgme.PubkeyAlgo.RSA)
        self.assertEqual(subkey.length, 1024)

        self.assertEqual(key.can_sign, True)
        self.assertEqual(key.can_encrypt, False)

        # The only UID available matches the given parameters.
        [uid] = key.uids
        self.assertEqual(uid.name, 'Testing')
        self.assertEqual(uid.comment, 'comment')
        self.assertEqual(uid.email, 'someone@example.com')

        # Finally check if the generated key can perform signatures.
        self.assertCanSign(key)

    def test_invalid_parameters(self) -> None:
        ctx = gpgme.Context()
        try:
            ctx.genkey('garbage parameters')
        except gpgme.GpgmeError as exc:
            self.assertTrue(hasattr(exc, "result"))
            result: gpgme.GenkeyResult = exc.result # type: ignore[assignment]
            self.assertEqual(result.primary, False)
            self.assertEqual(result.sub, False)
            self.assertEqual(result.fpr, None)
        else:
            self.fail("GpgmeError not raised")



# pygpgme - a Python wrapper for the gpgme library
# Copyright (C) 2006  James Henstridge
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from io import BytesIO
import os
from textwrap import dedent
from typing import Optional
import unittest

import gpgme
from tests.util import GpgHomeTestCase

class EncryptDecryptTestCase(GpgHomeTestCase):

    import_keys = ['key1.pub', 'key1.sec', 'key2.pub', 'key2.sec',
                   'signonly.pub', 'signonly.sec']

    def test_decrypt(self) -> None:
        ciphertext = BytesIO(dedent('''
            -----BEGIN PGP MESSAGE-----
            Version: GnuPG v1.4.1 (GNU/Linux)

            hQIMA6lSIdANy91kARAAtXjViihD6DVBAYGdVs0a2sMPGRXjIR5Tw1ONKx4MtKn+
            pydR+/+0rBRGaQXe/mMODA8gqADQjz7PcTMWBa7ja969+K2nw7j5DUIMatonQVMf
            vpc7ze5hovZ1jXYAgmmXdUzDmk8ZkpHaEc5mMMAHYKFn+mm37AFY5JUjg2Ae9k3H
            29t+pW+n9ncn/QBImW3oVslZ8Fza1xOIWZTUrmvtU0vELdlIxy+d945bvD9EhmTK
            zRrD5m8V1etWINO2tE1Xhd4lV1KxncHzWafXLB5BKprztTqFUXNPAfnucYIczDon
            5VvkOz3WAtl/93o85hUKhbgGK0dvU3m+bj620ZUE5oDpPB4l1CiO5RqUFYtyN4LF
            OSAceVOh7X764VLtpAzuOpNfTYgvzIFJqrFOZHlf3XJRdGdpJuxMe8BwhdLyDscx
            pap4QxajOUSUAeS45x6ERA7xHO0QOwXZNzoxiOt9KRaoIhEacu70A9xRcGNJfZE2
            3z/AEMKr2CK6ny9/S8UQEhNvn1/gYfSXakFjWjM6PUXJSnz8WGjpFKKITpex3WBz
            m/X8bKgG3fT92zqJdYocrl4wgz4Dt3+KirnGG4gITxaEYpTT0y6l6NAO60De0oRh
            yqk+ulj2pvAlA82Ph0vTGZ9uOQGbrfN+NhwsG0HMNq+vmjShS1jJbSfoEt1AAIPS
            RwGMq7SDk/V7nhKqZcxeIWdtRIgFvEf0KljthkOZxT/GozNjQclak7Kt69K2qyvt
            XUfidNAhuOmeZ4pF31qSFiAnKRbzjzHp
            =1s5N
            -----END PGP MESSAGE-----
            ''').encode('ASCII'))
        plaintext = BytesIO()
        ctx = gpgme.Context()
        ctx.decrypt(ciphertext, plaintext)
        self.assertEqual(plaintext.getvalue(), b'hello world\n')

    def test_decrypt_verify(self) -> None:
        ciphertext = BytesIO(dedent('''
            -----BEGIN PGP MESSAGE-----
            Version: GnuPG v1.4.1 (GNU/Linux)

            hQIMA6lSIdANy91kAQ/9GGQxL/OWvxrTchSMmIhsvwONJNlFE5cMIC0xejY4eN+t
            HtTg8V1fWXLRw7WY6FNFLeoR2hzqaZWw15lU55TmSJfJmK2mdUZu/IhSpCUFMEFW
            ZQpxslKq7N+S8NZHgq1WG32Ar1auOEflBQUMhj7sRSAtkvU7fWrTwf4Q4mcIV68P
            LiAAQoKxXNNVam9+EV/b3kx3bnJPKTn+ArpJf5Im+5XOGOeu9Ll0QUTbicgFhfpR
            esR6dKI/Ji5FGIu01kYNrDjDeMcJuzI52kNNoT+GJ72R+Gp4bZk2ycd+eVo3eeUW
            klO8K+7E5bd5ni+1H+ZWbVp9bn7Q++mFP6Mruv+v9Di5mvFXxMoFuB/8NzcilFVt
            h5VOexW1OaZk2bMp9bXVja/N7Y1oAADhINk0feaKkwYVOBJU9kJtL2O1WQui85Q3
            2dsL0YRJiR6mXesTezglZO44gsVAvCH8RUCtBnfEazfBg4jhcCHy6ooDgd0M4vcw
            xG4U7IyDU5xyLi9QrTaSg5LzzwNFqb5k/lTemZw3ob3uwZinWewASLwn5N5OPVRs
            gFT0eL0TfvDzHURsM/7QDvq9HX6JS7buyOlr5cZAsdSvm0FyE6YOkSvZR2jwp3vV
            jfs7RHjq9V7jzPVVKHnWEDoJfchkT/3KyMRCIM/ukBk9MwTZTIJRhjTA2Xd4kWTS
            kQEaU/OjumXPtw/T1pUH23nAkVssHsj8qgtxkFSmG/wrwNmfYx4tDhvgsHMJhar9
            hqQKBMsGmLD6RNWKhF/LryNBKI2IRgJabKKYbbOsydom/hw8ZF4aWaZTcCBMoBB2
            nhOi8WEIeWp93FGfHBa60nSBNGwgt24NmoFaXMjnCrJY/yK0L0MAajUC150OhtvG
            OSk=
            =fl3U
            -----END PGP MESSAGE-----
            ''').encode('ASCII'))
        plaintext = BytesIO()
        ctx = gpgme.Context()
        sigs = ctx.decrypt_verify(ciphertext, plaintext)
        self.assertEqual(plaintext.getvalue(), b'hello world\n')
        self.assertEqual(len(sigs), 1)
        self.assertEqual(sigs[0].summary, 0)
        self.assertEqual(sigs[0].fpr,
                         'E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        self.assertEqual(sigs[0].status, None)
        self.assertEqual(sigs[0].notations, [])
        self.assertEqual(sigs[0].timestamp, 1138049495)
        self.assertEqual(sigs[0].exp_timestamp, 0)
        self.assertEqual(sigs[0].wrong_key_usage, False)
        self.assertEqual(sigs[0].validity, gpgme.Validity.UNKNOWN)
        self.assertEqual(sigs[0].validity_reason, None)

    def test_encrypt(self) -> None:
        plaintext = BytesIO(b'Hello World\n')
        ciphertext = BytesIO()
        ctx = gpgme.Context()
        recipient = ctx.get_key('93C2240D6B8AA10AB28F701D2CF46B7FC97E6B0F')
        ctx.encrypt([recipient], gpgme.EncryptFlags.ALWAYS_TRUST,
                    plaintext, ciphertext)

        # rewind ciphertext buffer, and try to decrypt:
        ciphertext.seek(0)
        plaintext = BytesIO()
        ctx.decrypt(ciphertext, plaintext)
        self.assertEqual(plaintext.getvalue(), b'Hello World\n')

    def test_encrypt_armor(self) -> None:
        plaintext = BytesIO(b'Hello World\n')
        ciphertext = BytesIO()
        ctx = gpgme.Context()
        ctx.armor = True
        recipient = ctx.get_key('93C2240D6B8AA10AB28F701D2CF46B7FC97E6B0F')
        ctx.encrypt([recipient], gpgme.EncryptFlags.ALWAYS_TRUST,
                    plaintext, ciphertext)

        # rewind ciphertext buffer, and try to decrypt:
        ciphertext.seek(0)
        plaintext = BytesIO()
        ctx.decrypt(ciphertext, plaintext)
        self.assertEqual(plaintext.getvalue(), b'Hello World\n')

    def test_encrypt_symmetric(self) -> None:
        plaintext = BytesIO(b'Hello World\n')
        ciphertext = BytesIO()
        def passphrase(uid_hint: Optional[str], passphrase_info: Optional[str], prev_was_bad: bool, fd: int) -> None:
            os.write(fd, b'Symmetric passphrase\n')
        ctx = gpgme.Context()
        ctx.armor = True
        ctx.passphrase_cb = passphrase
        ctx.encrypt(None, 0, plaintext, ciphertext)
        self.assertTrue(
            ciphertext.getvalue().startswith(b'-----BEGIN PGP MESSAGE-----'))

        # Rewind ciphertext buffer and try to decrypt it:
        ciphertext.seek(0)
        plaintext = BytesIO()
        ctx.decrypt(ciphertext, plaintext)
        self.assertEqual(plaintext.getvalue(), b'Hello World\n')

    def test_encrypt_sign(self) -> None:
        plaintext = BytesIO(b'Hello World\n')
        ciphertext = BytesIO()
        ctx = gpgme.Context()
        ctx.armor = True
        signer = ctx.get_key('E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        recipient = ctx.get_key('93C2240D6B8AA10AB28F701D2CF46B7FC97E6B0F')
        ctx.signers = [signer]
        new_sigs = ctx.encrypt_sign([recipient], gpgme.EncryptFlags.ALWAYS_TRUST,
                                    plaintext, ciphertext)

        self.assertEqual(len(new_sigs), 1)
        self.assertEqual(new_sigs[0].type, gpgme.SigMode.NORMAL)
        self.assertEqual(new_sigs[0].fpr,
                        'E79A842DA34A1CA383F64A1546BB55F0885C65A4')

        # rewind ciphertext buffer, and try to decrypt:
        ciphertext.seek(0)
        plaintext = BytesIO()
        sigs = ctx.decrypt_verify(ciphertext, plaintext)
        self.assertEqual(plaintext.getvalue(), b'Hello World\n')
        self.assertEqual(len(sigs), 1)
        self.assertEqual(sigs[0].summary, 0)
        self.assertEqual(sigs[0].fpr,
                         'E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        self.assertEqual(sigs[0].status, None)
        self.assertEqual(sigs[0].wrong_key_usage, False)
        self.assertEqual(sigs[0].validity, gpgme.Validity.UNKNOWN)
        self.assertEqual(sigs[0].validity_reason, None)

    def test_encrypt_to_signonly(self) -> None:
        plaintext = BytesIO(b'Hello World\n')
        ciphertext = BytesIO()
        ctx = gpgme.Context()
        recipient = ctx.get_key('15E7CE9BF1771A4ABC550B31F540A569CB935A42')
        try:
            ctx.encrypt([recipient], gpgme.EncryptFlags.ALWAYS_TRUST,
                        plaintext, ciphertext)
        except gpgme.GpgmeError as exc:
            self.assertEqual(exc.args[0], gpgme.ErrSource.GPGME)
            self.assertEqual(exc.args[1], gpgme.ErrCode.UNUSABLE_PUBKEY)
        else:
            self.fail('gpgme.GpgmeError not raised')



# pygpgme - a Python wrapper for the gpgme library
# Copyright (C) 2006  James Henstridge
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
from typing import Optional
import unittest

import gpgme
from tests.util import GpgHomeTestCase

class ContextTestCase(GpgHomeTestCase):

    def test_constructor(self) -> None:
        ctx = gpgme.Context()

    def test_protocol(self) -> None:
        ctx = gpgme.Context()
        self.assertEqual(ctx.protocol, gpgme.Protocol.OpenPGP)
        ctx.protocol = gpgme.Protocol.CMS
        self.assertEqual(ctx.protocol, gpgme.Protocol.CMS)
        ctx.protocol = gpgme.Protocol.OpenPGP
        self.assertEqual(ctx.protocol, gpgme.Protocol.OpenPGP)

        # check error on setting to invalid protocol value
        def set_protocol(ctx: gpgme.Context, value: gpgme.Protocol) -> None:
            ctx.protocol = value
        self.assertRaises(gpgme.GpgmeError, set_protocol, ctx, 999)

        def del_protocol(ctx: gpgme.Context) -> None:
            del ctx.protocol
        self.assertRaises(AttributeError, del_protocol, ctx)

    def test_armor(self) -> None:
        ctx = gpgme.Context()
        self.assertEqual(ctx.armor, False)
        ctx.armor = True
        self.assertEqual(ctx.armor, True)
        ctx.armor = False
        self.assertEqual(ctx.armor, False)

        def del_armor(ctx: gpgme.Context) -> None:
            del ctx.armor
        self.assertRaises(AttributeError, del_armor, ctx)

    def test_textmode(self) -> None:
        ctx = gpgme.Context()
        self.assertEqual(ctx.textmode, False)
        ctx.textmode = True
        self.assertEqual(ctx.textmode, True)
        ctx.textmode = False
        self.assertEqual(ctx.textmode, False)

        def del_textmode(ctx: gpgme.Context) -> None:
            del ctx.textmode
        self.assertRaises(AttributeError, del_textmode, ctx)

    def test_offline(self) -> None:
        ctx = gpgme.Context()
        self.assertEqual(ctx.offline, False)
        ctx.offline = True
        self.assertEqual(ctx.offline, True)
        ctx.offline = False
        self.assertEqual(ctx.offline, False)
        with self.assertRaises(AttributeError):
            del ctx.offline

    def test_include_certs(self) -> None:
        ctx = gpgme.Context()
        self.assertEqual(ctx.include_certs, -256)
        ctx.include_certs = 2
        self.assertEqual(ctx.include_certs, 2)

        def del_include_certs(ctx: gpgme.Context) -> None:
            del ctx.include_certs
        self.assertRaises(AttributeError, del_include_certs, ctx)

    def test_keylist_mode(self) -> None:
        ctx = gpgme.Context()
        self.assertEqual(ctx.keylist_mode, gpgme.KeylistMode.LOCAL)
        ctx.keylist_mode = gpgme.KeylistMode.EXTERN
        self.assertEqual(ctx.keylist_mode, gpgme.KeylistMode.EXTERN)
        ctx.keylist_mode = gpgme.KeylistMode.LOCAL | gpgme.KeylistMode.EXTERN
        self.assertEqual(ctx.keylist_mode,
                         gpgme.KeylistMode.LOCAL | gpgme.KeylistMode.EXTERN)

        def del_keylist_mode(ctx: gpgme.Context) -> None:
            del ctx.keylist_mode
        self.assertRaises(AttributeError, del_keylist_mode, ctx)

    def test_passphrase_cb(self) -> None:
        ctx = gpgme.Context()
        def passphrase_cb(uid_hint: Optional[str], passphrase_info: Optional[str], prev_was_bad: bool, fd: int) -> None:
            pass
        self.assertEqual(ctx.passphrase_cb, None)
        ctx.passphrase_cb = passphrase_cb
        self.assertEqual(ctx.passphrase_cb, passphrase_cb)
        ctx.passphrase_cb = None
        self.assertEqual(ctx.passphrase_cb, None)
        ctx.passphrase_cb = passphrase_cb
        del ctx.passphrase_cb
        self.assertEqual(ctx.passphrase_cb, None)

    def test_progress_cb(self) -> None:
        ctx = gpgme.Context()
        def progress_cb(what: Optional[str], type: int, current: int, total: int) -> None:
            pass
        self.assertEqual(ctx.progress_cb, None)
        ctx.progress_cb = progress_cb
        self.assertEqual(ctx.progress_cb, progress_cb)
        ctx.progress_cb = None
        self.assertEqual(ctx.progress_cb, None)
        ctx.progress_cb = progress_cb
        del ctx.progress_cb
        self.assertEqual(ctx.progress_cb, None)

    def test_sender(self) -> None:
        ctx = gpgme.Context()
        self.assertEqual(ctx.sender, None)
        ctx.sender = 'test@example.org'
        self.assertEqual(ctx.sender, 'test@example.org')
        ctx.sender = 'Test <test@example.org>'
        self.assertEqual(ctx.sender, 'test@example.org')
        ctx.sender = None
        self.assertEqual(ctx.sender, None)
        with self.assertRaises(AttributeError):
            del ctx.sender

    def test_get_engine_info(self) -> None:
        ctx = gpgme.Context()
        for info in ctx.get_engine_info():
            self.assertIsInstance(info, gpgme.EngineInfo)
            self.assertIsInstance(info.protocol, gpgme.Protocol)
            self.assertIsInstance(info.file_name, (str, type(None)))
            self.assertIsInstance(info.version, (str, type(None)))
            self.assertIsInstance(info.req_version, (str, type(None)))
            self.assertIsInstance(info.home_dir, (str, type(None)))

    def test_set_engine_info(self) -> None:
        # Add a key using the default $GNUPGHOME based keyring.
        ctx = gpgme.Context()
        with self.keyfile('key1.pub') as fp:
            ctx.import_(fp)

        # If we set $GNUPGHOME to a dummy value, we can't read in the
        # keywe just loaded.
        os.environ['GNUPGHOME'] = '/no/such/dir'
        ctx = gpgme.Context()
        self.assertRaises(gpgme.GpgmeError, ctx.get_key,
                          'E79A842DA34A1CA383F64A1546BB55F0885C65A4')

        # But if we configure the context using set_engine_info(), it
        # will find the key.
        ctx = gpgme.Context()
        ctx.set_engine_info(gpgme.Protocol.OpenPGP, None, self._gpghome)
        key = ctx.get_key('E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        self.assertTrue(key)

        info = [info for info in ctx.get_engine_info()
                if info.protocol == gpgme.Protocol.OpenPGP].pop()
        self.assertEqual(info.home_dir, self._gpghome)



# pygpgme - a Python wrapper for the gpgme library
# Copyright (C) 2006  James Henstridge
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
from io import BytesIO
from textwrap import dedent
from typing import Optional
import unittest

import gpgme
from tests.util import GpgHomeTestCase

class ProgressTestCase(GpgHomeTestCase):

    import_keys = ['key1.pub', 'key1.sec']

    def progress_cb(self, what: Optional[str], type_: int, current: int, total: int) -> None:
        self.progress_cb_called = True

    def test_sign_with_progress_cb(self) -> None:
        ctx = gpgme.Context()
        key = ctx.get_key('E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        ctx.signers = [key]
        ctx.progress_cb = self.progress_cb
        plaintext = BytesIO(b'Hello World\n')
        signature = BytesIO()

        self.progress_cb_called = False
        new_sigs = ctx.sign(plaintext, signature, gpgme.SigMode.CLEAR)

        # ensure that progress_cb has been run
        self.assertEqual(self.progress_cb_called, True)

        self.assertEqual(new_sigs[0].type, gpgme.SigMode.CLEAR)
        self.assertEqual(new_sigs[0].fpr,
                        'E79A842DA34A1CA383F64A1546BB55F0885C65A4')



# pygpgme - a Python wrapper for the gpgme library
# Copyright (C) 2006  James Henstridge
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from io import BytesIO
import os
from textwrap import dedent
from typing import Optional
import unittest

import gpgme
from tests.util import GpgHomeTestCase

class PassphraseTestCase(GpgHomeTestCase):

    import_keys = ['passphrase.pub', 'passphrase.sec']

    uid_hint: Optional[str]
    passphrase_info: Optional[str]
    prev_was_bad: Optional[bool]

    def test_sign_without_passphrase_cb(self) -> None:
        ctx = gpgme.Context()
        key = ctx.get_key('EFB052B4230BBBC51914BCBB54DCBBC8DBFB9EB3')
        ctx.signers = [key]
        plaintext = BytesIO(b'Hello World\n')
        signature = BytesIO()

        try:
            new_sigs = ctx.sign(plaintext, signature, gpgme.SigMode.CLEAR)
        except gpgme.GpgmeError as exc:
            self.assertEqual(exc.args[0], gpgme.ErrSource.GPGME)
            self.assertEqual(exc.args[1], gpgme.ErrCode.GENERAL)
        else:
            self.fail('gpgme.GpgmeError not raised')

    def passphrase_cb(self, uid_hint: Optional[str], passphrase_info: Optional[str], prev_was_bad: bool, fd: int) -> None:
        self.uid_hint = uid_hint
        self.passphrase_info = passphrase_info
        self.prev_was_bad = prev_was_bad
        os.write(fd, b'test\n')

    def test_sign_with_passphrase_cb(self) -> None:
        ctx = gpgme.Context()
        key = ctx.get_key('EFB052B4230BBBC51914BCBB54DCBBC8DBFB9EB3')
        ctx.signers = [key]
        ctx.passphrase_cb = self.passphrase_cb
        plaintext = BytesIO(b'Hello World\n')
        signature = BytesIO()

        self.uid_hint = None
        self.passphrase_info = None
        self.prev_was_bad = None
        new_sigs = ctx.sign(plaintext, signature, gpgme.SigMode.CLEAR)

        # ensure that passphrase_cb has been run, and the data it was passed
        self.assertEqual(self.uid_hint,
            '54DCBBC8DBFB9EB3 Passphrase (test) <passphrase@example.org>')
        self.assertEqual(self.passphrase_info,
            '54DCBBC8DBFB9EB3 54DCBBC8DBFB9EB3 17 0')
        self.assertEqual(self.prev_was_bad, False)

        self.assertEqual(new_sigs[0].type, gpgme.SigMode.CLEAR)
        self.assertEqual(new_sigs[0].fpr,
                        'EFB052B4230BBBC51914BCBB54DCBBC8DBFB9EB3')



# pygpgme - a Python wrapper for the gpgme library
# Copyright (C) 2006  James Henstridge
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from io import BytesIO
import os
from typing import Optional
import unittest

import gpgme
import gpgme.editutil
from tests.util import GpgHomeTestCase

class EditKeyTestCase(GpgHomeTestCase):

    import_keys = ['key1.pub', 'key1.sec', 'key2.pub',
                   'signonly.pub', 'signonly.sec']

    status: Optional[gpgme.Status]
    args: Optional[str]

    def edit_quit_cb(self, status: gpgme.Status, args: Optional[str], fd: int) -> None:
        if status in [gpgme.Status.EOF, gpgme.Status.GOT_IT, gpgme.Status.KEY_CONSIDERED]:
            return
        self.status = status
        self.args = args
        os.write(fd, b'quit\n')

    def test_edit_quit(self) -> None:
        ctx = gpgme.Context()
        key = ctx.get_key('E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        output = BytesIO()

        self.status = None
        self.args = None
        with self.assertWarns(DeprecationWarning):
            ctx.edit(key, self.edit_quit_cb, output)

        self.assertEqual(self.status, gpgme.Status.GET_LINE)
        self.assertEqual(self.args, 'keyedit.prompt')

    def test_edit_ownertrust(self) -> None:
        ctx = gpgme.Context()
        key = ctx.get_key('93C2240D6B8AA10AB28F701D2CF46B7FC97E6B0F')
        self.assertEqual(key.owner_trust, gpgme.Validity.UNKNOWN)

        # try setting each validity:
        for trust in [gpgme.Validity.NEVER,
                      gpgme.Validity.MARGINAL,
                      gpgme.Validity.FULL,
                      gpgme.Validity.ULTIMATE]:
            with self.assertWarns(DeprecationWarning):
                gpgme.editutil.edit_trust(ctx, key, trust)
            key = ctx.get_key('93C2240D6B8AA10AB28F701D2CF46B7FC97E6B0F')
            self.assertEqual(key.owner_trust, trust)

    def test_edit_sign(self) -> None:
        ctx = gpgme.Context()
        # we set the keylist mode so we can see signatures
        ctx.keylist_mode = gpgme.KeylistMode.SIGS
        ctx.signers = [ctx.get_key('15E7CE9BF1771A4ABC550B31F540A569CB935A42')]
        key = ctx.get_key('E79A842DA34A1CA383F64A1546BB55F0885C65A4')

        # check that there are no signatures from 0xCB935A42
        for uid in key.uids:
            sigs = [sig for sig in uid.signatures
                    if sig.keyid == 'F540A569CB935A42']
            self.assertEqual(len(sigs), 0)

        with self.assertWarns(DeprecationWarning):
            gpgme.editutil.edit_sign(ctx, key, check=0)
        key = ctx.get_key('E79A842DA34A1CA383F64A1546BB55F0885C65A4')

        # check that there is a signature from 0xCB935A42 on each UID
        for uid in key.uids:
            sigs = [sig for sig in uid.signatures
                    if sig.keyid == 'F540A569CB935A42']
            self.assertEqual(len(sigs), 1)



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

import os

import gpgme


project = 'PyGPGME'
copyright = '2024, PyGPGME contributors'
author = 'PyGPGME contributors'

release = gpgme.__version__

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.napoleon',
    'sphinx.ext.todo',
]

templates_path = ['_templates']
exclude_patterns = []

autodoc_default_options = {
    'member-order': 'groupwise',
}

# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = 'sphinx_rtd_theme'

html_context = {
    'display_github': True,
    'github_user': 'jhenstridge',
    'github_repo': 'pygpgme',
    'github_version': os.environ.get('READTHEDOCS_GIT_IDENTIFIER', os.environ.get('GITHUB_REF', 'main')),
    'conf_py_path': '/docs/source/',
}



# pygpgme - a Python wrapper for the gpgme library
# Copyright (C) 2006  James Henstridge
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import unittest

import gpgme
from tests.util import GpgHomeTestCase

class DeleteTestCase(GpgHomeTestCase):

    import_keys = ['key1.pub', 'key1.sec', 'key2.pub']

    def test_delete_public_key(self) -> None:
        ctx = gpgme.Context()
        # key2
        key = ctx.get_key('93C2240D6B8AA10AB28F701D2CF46B7FC97E6B0F')
        ctx.delete(key)

        # check that it is deleted
        with self.assertRaises(gpgme.GpgmeError):
            ctx.get_key('93C2240D6B8AA10AB28F701D2CF46B7FC97E6B0F')

    def test_delete_public_key_with_secret_key(self) -> None:
        ctx = gpgme.Context()
        # key1
        key = ctx.get_key('E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        with self.assertRaises(gpgme.GpgmeError):
            ctx.delete(key)

    def test_delete_secret_key(self) -> None:
        ctx = gpgme.Context()
        # key2
        key = ctx.get_key('E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        ctx.delete(key, gpgme.Delete.ALLOW_SECRET | gpgme.Delete.FORCE)
        # check that it is deleted
        with self.assertRaises(gpgme.GpgmeError):
            ctx.get_key('E79A842DA34A1CA383F64A1546BB55F0885C65A4')

    def test_delete_secret_force(self) -> None:
        pass

    def test_delete_non_existant(self) -> None:
        ctx = gpgme.Context()
        # key2
        key = ctx.get_key('93C2240D6B8AA10AB28F701D2CF46B7FC97E6B0F')
        ctx.delete(key)

        # delete it again
        try:
            ctx.delete(key)
        except gpgme.GpgmeError as exc:
            self.assertEqual(exc.args[0], gpgme.ErrSource.GPGME)
            self.assertEqual(exc.args[1], gpgme.ErrCode.NO_PUBKEY)
        else:
            self.fail('gpgme.GpgmeError was not raised')

    def test_constants(self) -> None:
        # Old releases used a boolean for the second argument to
        # delete. Check that the boolean constants match the same flag
        # values:
        self.assertEqual(True, gpgme.Delete.ALLOW_SECRET)



# pygpgme - a Python wrapper for the gpgme library
# Copyright (C) 2006  James Henstridge
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import shutil
import subprocess
import tempfile
from typing import BinaryIO
import unittest

import gpgme

__all__ = ['GpgHomeTestCase']

keydir = os.path.join(os.path.dirname(__file__), 'keys')

class GpgHomeTestCase(unittest.TestCase):

    import_keys: list[str] = []

    def keyfile(self, key: str) -> BinaryIO:
        return open(os.path.join(keydir, key), 'rb')

    def setUp(self) -> None:
        self._gpghome = tempfile.mkdtemp(prefix='tmp.gpghome')
        os.environ['GNUPGHOME'] = self._gpghome
        with open(os.path.join(self._gpghome, 'gpg.conf'), 'w') as fp:
            fp.write('pinentry-mode loopback\n')
            fp.write('keyserver hkp://keyserver.invalid\n')
        with open(os.path.join(self._gpghome, 'gpg-agent.conf'), 'w') as fp:
            fp.write('allow-loopback-pinentry\n')
        subprocess.check_call(['gpg-connect-agent', '/bye'],
                              stdout=subprocess.DEVNULL,
                              stderr=subprocess.DEVNULL)

        # import requested keys into the keyring
        ctx = gpgme.Context()
        for key in self.import_keys:
            with self.keyfile(key) as fp:
                ctx.import_(fp)

    def tearDown(self) -> None:
        # May fail if the agent is not currently running.
        subprocess.call(['gpg-connect-agent', 'KILLAGENT', '/bye'],
                              stdout=subprocess.DEVNULL,
                              stderr=subprocess.DEVNULL)
        del os.environ['GNUPGHOME']
        shutil.rmtree(self._gpghome, ignore_errors=True)



# pygpgme - a Python wrapper for the gpgme library
# Copyright (C) 2006  James Henstridge
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from io import BytesIO
from textwrap import dedent
import unittest

import gpgme
from tests.util import GpgHomeTestCase

class SignVerifyTestCase(GpgHomeTestCase):

    import_keys = ['key1.pub', 'key1.sec', 'key2.pub', 'key2.sec',
                   'signonly.pub', 'signonly.sec']

    def test_verify_normal(self) -> None:
        signature = BytesIO(dedent('''
            -----BEGIN PGP MESSAGE-----
            Version: GnuPG v1.4.1 (GNU/Linux)

            owGbwMvMwCTotjv0Q0dM6hLG00JJDM7nNx31SM3JyVcIzy/KSeHqsGdmBQvCVAky
            pR9hmGfw0qo3bfpWZwun5euYAsUcVkyZMJlhfvkU6UBjD8WF9RfeND05zC/TK+H+
            EQA=
            =HCW0
            -----END PGP MESSAGE-----
            ''').encode('ASCII'))
        plaintext = BytesIO()
        ctx = gpgme.Context()
        sigs = ctx.verify(signature, None, plaintext)

        self.assertEqual(plaintext.getvalue(), b'Hello World\n')
        self.assertEqual(len(sigs), 1)
        self.assertEqual(sigs[0].summary, 0)
        self.assertEqual(sigs[0].fpr,
                         'E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        self.assertEqual(sigs[0].status, None)
        self.assertEqual(sigs[0].notations, [])
        self.assertEqual(sigs[0].timestamp, 1137685189)
        self.assertEqual(sigs[0].exp_timestamp, 0)
        self.assertEqual(sigs[0].wrong_key_usage, False)
        self.assertEqual(sigs[0].validity, gpgme.Validity.UNKNOWN)
        self.assertEqual(sigs[0].validity_reason, None)
        self.assertEqual(sigs[0].pubkey_algo, gpgme.PubkeyAlgo.DSA)
        self.assertEqual(sigs[0].hash_algo, gpgme.HashAlgo.SHA1)

    def test_verify_detached(self) -> None:
        signature = BytesIO(dedent('''
            -----BEGIN PGP SIGNATURE-----
            Version: GnuPG v1.4.1 (GNU/Linux)

            iD8DBQBDz7ReRrtV8IhcZaQRAtuUAJwMiJeS5QPohToxA3+vp+z5c3jr1wCdHhGP
            hhSTiguzgSYNwKSuV6SLGOM=
            =dyZS
            -----END PGP SIGNATURE-----
            ''').encode('ASCII'))
        signed_text = BytesIO(b'Hello World\n')
        ctx = gpgme.Context()
        sigs = ctx.verify(signature, signed_text, None)

        self.assertEqual(len(sigs), 1)
        self.assertEqual(sigs[0].summary, 0)
        self.assertEqual(sigs[0].fpr,
                         'E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        self.assertEqual(sigs[0].status, None)
        self.assertEqual(sigs[0].notations, [])
        self.assertEqual(sigs[0].timestamp, 1137685598)
        self.assertEqual(sigs[0].exp_timestamp, 0)
        self.assertEqual(sigs[0].wrong_key_usage, False)
        self.assertEqual(sigs[0].validity, gpgme.Validity.UNKNOWN)
        self.assertEqual(sigs[0].validity_reason, None)
        self.assertEqual(sigs[0].pubkey_algo, gpgme.PubkeyAlgo.DSA)
        self.assertEqual(sigs[0].hash_algo, gpgme.HashAlgo.SHA1)

    def test_verify_clearsign(self) -> None:
        signature = BytesIO(dedent('''
            -----BEGIN PGP SIGNED MESSAGE-----
            Hash: SHA1

            Hello World
            -----BEGIN PGP SIGNATURE-----
            Version: GnuPG v1.4.1 (GNU/Linux)

            iD8DBQFDz7DiRrtV8IhcZaQRAjuYAJ43/NhhNHx+gzGBUqtIK5LpENTCGgCfV3aO
            ZTFlGRyKN26HccsC6ZWcPUQ=
            =kZ2c
            -----END PGP SIGNATURE-----
            ''').encode('ASCII'))
        plaintext = BytesIO()
        ctx = gpgme.Context()
        sigs = ctx.verify(signature, None, plaintext)

        self.assertEqual(plaintext.getvalue(), b'Hello World\n')
        self.assertEqual(len(sigs), 1)
        self.assertEqual(sigs[0].summary, 0)
        self.assertEqual(sigs[0].fpr,
                         'E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        self.assertEqual(sigs[0].status, None)
        self.assertEqual(sigs[0].notations, [])
        self.assertEqual(sigs[0].timestamp, 1137684706)
        self.assertEqual(sigs[0].exp_timestamp, 0)
        self.assertEqual(sigs[0].wrong_key_usage, False)
        self.assertEqual(sigs[0].validity, gpgme.Validity.UNKNOWN)
        self.assertEqual(sigs[0].validity_reason, None)
        self.assertEqual(sigs[0].pubkey_algo, gpgme.PubkeyAlgo.DSA)
        self.assertEqual(sigs[0].hash_algo, gpgme.HashAlgo.SHA1)

    def test_verify_multiple_sigs(self) -> None:
        signature = BytesIO(dedent('''
            -----BEGIN PGP SIGNED MESSAGE-----
            Hash: SHA1

            Hello World
            -----BEGIN PGP SIGNATURE-----
            Version: GnuPG v1.4.1 (GNU/Linux)

            iD8DBQFDz7V9RrtV8IhcZaQRAia/AJ9eC/Q3pssWW9PWckQ3+1kbiIiEVQCfSeFv
            7SlUCFJOs/sfl+EtaOafgQGJAhUDBQFDz7V9LPRrf8l+aw8BAia/EAClI1X/hL38
            6NeOnMD6zXNm7r20Qkpp7PT63PqUa9dU1P+Ha2Uju5C2jBVYouDOpHnEsw3AqItl
            M0y6xiBAbXbdv0K2OdX8/290g/uODQE/oRGu+YtIh8HcY9N1JmzYw6msRO1LD/Oo
            xVqfyJiPx+Ol3juAuVqggBzQQmhQpZ7MfHcZSIWxYtRZNlCGYp2lUVae7fJlrJc8
            DvTkGSkdqBRoDqy0rKcdXRuExXyq081m7bli2sMvImejmEsqyMcbZrkW69v+/BQD
            Tki8tEkxINw1YHhcBDI0KAn3SuynY+i132oU2qJWQF3ZBRqEbD0IxfakPSZyhJKj
            sxk38VHgA+5r/QKRs+4n3z09yFqNIWpnvVVZ2iMfKhHtKd1nNq6tOzHiQrmdSdyK
            dwRaRm4Zt0hWT8v+CXX/RPK5xGL3FCZQs7VTO0ANHR7cIS+v3ChaHO6naQSBQMrW
            7l69hTh009LFIKlYJ+7ZBS2pySkvHmEzJKl4Ko4UfOeD2xDsq5nHhi/AJ7TXtHCo
            TLo8OwJvfiW6Fa9zzu6IkerhQlZrvbLOkmBpuyFo0UEuM/89bquaZ3GoEj3hePsZ
            nD9LtsgsjkFV1jZQ4n/wM3jolo0aA4+ZEBCgw9XJUSZ67m+jvFNBvZtDqWnbQWxe
            FsW3EQWNlQnwkn2lic51Cdp3w7yPH5CKfw==
            =0A7N
            -----END PGP SIGNATURE-----
            ''').encode('ASCII'))
        plaintext = BytesIO()
        ctx = gpgme.Context()
        sigs = ctx.verify(signature, None, plaintext)

        self.assertEqual(plaintext.getvalue(), b'Hello World\n')
        self.assertEqual(len(sigs), 2)
        self.assertEqual(sigs[0].summary, 0)
        self.assertEqual(sigs[0].fpr,
                         'E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        self.assertEqual(sigs[0].status, None)
        self.assertEqual(sigs[0].notations, [])
        self.assertEqual(sigs[0].timestamp, 1137685885)
        self.assertEqual(sigs[0].exp_timestamp, 0)
        self.assertEqual(sigs[0].wrong_key_usage, False)
        self.assertEqual(sigs[0].validity, gpgme.Validity.UNKNOWN)
        self.assertEqual(sigs[0].validity_reason, None)
        self.assertEqual(sigs[0].pubkey_algo, gpgme.PubkeyAlgo.DSA)
        self.assertEqual(sigs[0].hash_algo, gpgme.HashAlgo.SHA1)

        self.assertEqual(sigs[1].summary, 0)
        self.assertEqual(sigs[1].fpr,
                         '93C2240D6B8AA10AB28F701D2CF46B7FC97E6B0F')
        self.assertEqual(sigs[1].status, None)
        self.assertEqual(sigs[1].notations, [])
        self.assertEqual(sigs[1].timestamp, 1137685885)
        self.assertEqual(sigs[1].exp_timestamp, 0)
        self.assertEqual(sigs[1].wrong_key_usage, False)
        self.assertEqual(sigs[1].validity, gpgme.Validity.UNKNOWN)
        self.assertEqual(sigs[1].validity_reason, None)
        self.assertEqual(sigs[1].pubkey_algo, gpgme.PubkeyAlgo.RSA)
        self.assertEqual(sigs[1].hash_algo, gpgme.HashAlgo.SHA1)

    def test_verify_no_signature(self) -> None:
        signature = BytesIO(dedent('''
            -----BEGIN PGP SIGNED MESSAGE-----
            Hash: SHA1

            Hello World
            -----BEGIN PGP SIGNATURE-----
            -----END PGP SIGNATURE-----
            ''').encode('ASCII'))
        plaintext = BytesIO()
        ctx = gpgme.Context()
        sigs = ctx.verify(signature, None, plaintext)

        self.assertEqual(plaintext.getvalue(), b'')
        self.assertEqual(len(sigs), 0)

    def test_verify_bad_signature(self) -> None:
        signature = BytesIO(dedent('''
            -----BEGIN PGP SIGNED MESSAGE-----
            Hash: SHA1

            Hello World
            -----BEGIN PGP SIGNATURE-----
            Version: GnuPG v1.4.1 (GNU/Linux)

            iNhhNHx+gzGBUqtIK5LpENTCGgCfV3aO
            -----END PGP SIGNATURE-----
            ''').encode('ASCII'))
        plaintext = BytesIO()
        ctx = gpgme.Context()
        try:
            ctx.verify(signature, None, plaintext)
        except gpgme.GpgmeError as exc:
            self.assertEqual(exc.args[0], gpgme.ErrSource.GPGME)
            self.assertEqual(exc.args[1], gpgme.ErrCode.NO_DATA)
        else:
            self.fail('gpgme.GpgmeError not raised')

    def test_sign_normal(self) -> None:
        ctx = gpgme.Context()
        ctx.armor = False
        key = ctx.get_key('E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        ctx.signers = [key]
        plaintext = BytesIO(b'Hello World\n')
        signature = BytesIO()

        new_sigs = ctx.sign(plaintext, signature, gpgme.SigMode.NORMAL)
        self.assertEqual(len(new_sigs), 1)
        self.assertEqual(new_sigs[0].type, gpgme.SigMode.NORMAL)
        self.assertEqual(new_sigs[0].fpr,
                        'E79A842DA34A1CA383F64A1546BB55F0885C65A4')

        # now verify the signature
        signature.seek(0)
        plaintext = BytesIO()
        sigs = ctx.verify(signature, None, plaintext)
        self.assertEqual(plaintext.getvalue(), b'Hello World\n')
        self.assertEqual(len(sigs), 1)
        self.assertEqual(sigs[0].summary, 0)
        self.assertEqual(sigs[0].fpr,
                         'E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        self.assertEqual(sigs[0].status, None)
        self.assertEqual(sigs[0].wrong_key_usage, False)
        self.assertEqual(sigs[0].validity, gpgme.Validity.UNKNOWN)
        self.assertEqual(sigs[0].validity_reason, None)

    def test_sign_normal_armor(self) -> None:
        ctx = gpgme.Context()
        ctx.armor = True
        key = ctx.get_key('E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        ctx.signers = [key]
        plaintext = BytesIO(b'Hello World\n')
        signature = BytesIO()

        new_sigs = ctx.sign(plaintext, signature, gpgme.SigMode.NORMAL)
        self.assertEqual(len(new_sigs), 1)
        self.assertEqual(new_sigs[0].type, gpgme.SigMode.NORMAL)
        self.assertEqual(new_sigs[0].fpr,
                        'E79A842DA34A1CA383F64A1546BB55F0885C65A4')

        # now verify the signature
        signature.seek(0)
        plaintext = BytesIO()
        sigs = ctx.verify(signature, None, plaintext)
        self.assertEqual(plaintext.getvalue(), b'Hello World\n')
        self.assertEqual(len(sigs), 1)
        self.assertEqual(sigs[0].summary, 0)
        self.assertEqual(sigs[0].fpr,
                         'E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        self.assertEqual(sigs[0].status, None)
        self.assertEqual(sigs[0].wrong_key_usage, False)
        self.assertEqual(sigs[0].validity, gpgme.Validity.UNKNOWN)
        self.assertEqual(sigs[0].validity_reason, None)

    def test_sign_detatch(self) -> None:
        ctx = gpgme.Context()
        ctx.armor = True
        key = ctx.get_key('E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        ctx.signers = [key]
        plaintext = BytesIO(b'Hello World\n')
        signature = BytesIO()

        new_sigs = ctx.sign(plaintext, signature, gpgme.SigMode.DETACH)
        self.assertEqual(len(new_sigs), 1)
        self.assertEqual(new_sigs[0].type, gpgme.SigMode.DETACH)
        self.assertEqual(new_sigs[0].fpr,
                        'E79A842DA34A1CA383F64A1546BB55F0885C65A4')

        # now verify the signature
        signature.seek(0)
        plaintext.seek(0)
        sigs = ctx.verify(signature, plaintext, None)
        self.assertEqual(len(sigs), 1)
        self.assertEqual(sigs[0].summary, 0)
        self.assertEqual(sigs[0].fpr,
                         'E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        self.assertEqual(sigs[0].status, None)
        self.assertEqual(sigs[0].wrong_key_usage, False)
        self.assertEqual(sigs[0].validity, gpgme.Validity.UNKNOWN)
        self.assertEqual(sigs[0].validity_reason, None)

    def test_sign_clearsign(self) -> None:
        ctx = gpgme.Context()
        ctx.armor = True
        key = ctx.get_key('E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        ctx.signers = [key]
        plaintext = BytesIO(b'Hello World\n')
        signature = BytesIO()

        new_sigs = ctx.sign(plaintext, signature, gpgme.SigMode.CLEAR)
        self.assertEqual(len(new_sigs), 1)
        self.assertEqual(new_sigs[0].type, gpgme.SigMode.CLEAR)
        self.assertEqual(new_sigs[0].fpr,
                        'E79A842DA34A1CA383F64A1546BB55F0885C65A4')

        # now verify the signature
        signature.seek(0)
        plaintext = BytesIO()
        sigs = ctx.verify(signature, None, plaintext)
        self.assertEqual(plaintext.getvalue(), b'Hello World\n')
        self.assertEqual(len(sigs), 1)
        self.assertEqual(sigs[0].summary, 0)
        self.assertEqual(sigs[0].fpr,
                         'E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        self.assertEqual(sigs[0].status, None)
        self.assertEqual(sigs[0].wrong_key_usage, False)
        self.assertEqual(sigs[0].validity, gpgme.Validity.UNKNOWN)
        self.assertEqual(sigs[0].validity_reason, None)

    def test_sign_notations(self) -> None:
        ctx = gpgme.Context()
        key = ctx.get_key('E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        ctx.signers = [key]
        ctx.sig_notations = [
            gpgme.SigNotation('test@example.com', 'test value'),
            gpgme.SigNotation('unicode@example.com', '\xa7v1'),
            gpgme.SigNotation(None, 'http://example.com')]
        plaintext = BytesIO(b'Hello World\n')
        signature = BytesIO()

        new_sigs = ctx.sign(plaintext, signature, gpgme.SigMode.NORMAL)
        self.assertEqual(len(new_sigs), 1)
        self.assertEqual(new_sigs[0].fpr,
                        'E79A842DA34A1CA383F64A1546BB55F0885C65A4')

        # now verify the signature
        signature.seek(0)
        plaintext = BytesIO()
        sigs = ctx.verify(signature, None, plaintext)
        self.assertEqual(plaintext.getvalue(), b'Hello World\n')
        self.assertEqual(len(sigs), 1)
        self.assertEqual(sigs[0].fpr,
                         'E79A842DA34A1CA383F64A1546BB55F0885C65A4')
        notations = sorted(sigs[0].notations, key=lambda n: n.name or '')
        self.assertEqual(len(notations), 3)
        self.assertEqual(notations[0].name, None)
        self.assertEqual(notations[0].value, b'http://example.com')
        self.assertEqual(notations[0].flags, 0)
        self.assertEqual(notations[1].name, 'test@example.com')
        self.assertEqual(notations[1].value, 'test value')
        self.assertEqual(notations[1].flags, gpgme.SigNotationFlags.HUMAN_READABLE)
        self.assertEqual(notations[2].name, 'unicode@example.com')
        self.assertEqual(notations[2].value, '\xa7v1')
        self.assertEqual(notations[2].flags, gpgme.SigNotationFlags.HUMAN_READABLE)



import enum
from typing import (
    BinaryIO, Callable, Iterator, Literal, Optional, Sequence, Union, final)

@final
class Context:
    def __init__(self) -> None: ...
    def get_engine_info(self) -> Sequence[EngineInfo]: ...
    def set_engine_info(self, protocol: Protocol, file_name: Optional[str],
                        home_dir: Optional[str], /) -> None: ...
    def set_locale(self, category: int, value: Optional[str], /) -> None: ...
    def get_key(self, fingerprint: str, secret: bool = False, /) -> Key: ...
    def encrypt(self, recipients: Optional[Sequence[Key]],
                flags: EncryptFlags | Literal[0],
                plain: BinaryIO, cipher: BinaryIO, /) -> None: ...
    def encrypt_sign(self, recipients: Optional[Sequence[Key]],
                     flags: EncryptFlags | Literal[0],
                     plain: BinaryIO, cipher: BinaryIO, /) -> Sequence[NewSignature]: ...
    def decrypt(self, cipher: BinaryIO, plain: BinaryIO, /) -> None: ...
    def decrypt_verify(self, cipher: BinaryIO, plain: BinaryIO, /) -> Sequence[Signature]: ...
    def sign(self, plain: BinaryIO, sig: BinaryIO,
             sig_mode: SigMode = SigMode.NORMAL, /) -> Sequence[NewSignature]: ...
    def verify(self, sig: BinaryIO, signed_text: Optional[BinaryIO], plaintext: Optional[BinaryIO], /) -> Sequence[Signature]: ...
    def import_(self, keydata: BinaryIO, /) -> ImportResult: ...
    def import_keys(self, keys: Sequence[Key], /) -> ImportResult: ...
    def export(self, pattern: Union[None, str, Sequence[str]],
               keydata: BinaryIO,
               export_mode: ExportMode | Literal[0] = 0, /) -> None: ...
    def export_keys(self, keys: Sequence[Key],
                    keydata: BinaryIO,
                    export_mode: ExportMode | Literal[0] = 0, /) -> None: ...
    def genkey(self, params: Optional[str], pubkey: Optional[BinaryIO] = None,
               seckey: Optional[BinaryIO] = None, /) -> GenkeyResult: ...
    def delete(self, key: Key, flags: Delete | bool | Literal[0] = 0, /) -> None: ...
    def edit(self, key: Key, callback: Callable[[Status, Optional[str], int], None],
             out: BinaryIO, /) -> None: ...
    def card_edit(self, key: Key, callback: Callable[[Status, Optional[str], int], None],
                  out: BinaryIO, /) -> None: ...
    def keylist(self, pattern: Union[None, str, Sequence[str]] = None,
                secret_only: bool = False, /) -> Iterator[Key]: ...
    protocol: Protocol
    armor: bool
    textmode: bool
    offline: bool
    include_certs: int
    keylist_mode: KeylistMode | Literal[0]
    pinentry_mode: PinentryMode
    passphrase_cb: Optional[Callable[[Optional[str], Optional[str], bool, int], None]]
    progress_cb: Optional[Callable[[Optional[str], int, int, int], None]]
    signers: Sequence[Key]
    sig_notations: Sequence[SigNotation]
    sender: Optional[str]

@final
class EngineInfo:
    protocol: Protocol
    file_name: Optional[str]
    version: Optional[str]
    req_version: Optional[str]
    home_dir: Optional[str]

@final
class Key:
    revoked: bool
    expired: bool
    disabled: bool
    invalid: bool
    can_encrypt: bool
    can_sign: bool
    can_certify: bool
    secret: bool
    can_authenticate: bool
    protocol: Protocol
    issuer_serial: str
    issuer_name: str
    chain_id: str
    owner_trust: Validity
    subkeys: Sequence[Subkey]
    uids: Sequence[UserId]
    keylist_mode: KeylistMode | Literal[0]

@final
class Subkey:
    revoked: bool
    expired: bool
    disabled: bool
    invalid: bool
    can_encrypt: bool
    can_sign: bool
    can_certify: bool
    secret: bool
    can_authenticate: bool
    pubkey_algo: PubkeyAlgo
    length: int
    keyid: str
    fpr: str
    timestamp: int
    expires: int

@final
class UserId:
    revoked: bool
    invalid: bool
    validity: bool
    uid: Optional[str]
    name: Optional[str]
    email: Optional[str]
    comment: Optional[str]
    signatures: Sequence[KeySig]

@final
class KeySig:
    revoked: bool
    expired: bool
    invalid: bool
    exportable: bool
    pubkey_algo: PubkeyAlgo
    keyid: str
    timestamp: int
    expires: int
    status: Optional[GpgmeError]
    uid: Optional[str]
    name: Optional[str]
    email: Optional[str]
    comment: Optional[str]
    sig_class: int

@final
class NewSignature:
    type: SigMode
    pubkey_algo: PubkeyAlgo
    hash_algo: HashAlgo
    timestamp: int
    fpr: Optional[str]
    sig_class: int

@final
class Signature:
    summary: Sigsum | Literal[0]
    fpr: Optional[str]
    status: Optional[GpgmeError]
    notations: Sequence[SigNotation]
    timestamp: int
    exp_timestamp: int
    wrong_key_usage: bool
    validity: Validity
    validity_reason: Optional[GpgmeError]
    pubkey_algo: PubkeyAlgo
    hash_algo: HashAlgo

@final
class SigNotation:
    def __init__(self, name: Optional[str], value: str | bytes, flags: SigNotationFlags = SigNotationFlags.HUMAN_READABLE) -> None: ...
    name: Optional[str]
    value: str | bytes
    flags: SigNotationFlags | Literal[0]
    human_readable: bool
    critical: bool

@final
class ImportResult:
    considered: int
    no_user_id: int
    imported: int
    imported_rsa: int
    unchanged: int
    new_user_ids: int
    new_sub_keys: int
    new_signatures: int
    new_revocations: int
    secret_read: int
    secret_imported: int
    secret_unchanged: int
    skipped_new_keys: int
    not_imported: int
    imports: Sequence[tuple[Optional[str], Optional[GpgmeError], Import]]

@final
class GenkeyResult:
    primary: bool
    sub: bool
    fpr: str

@final
class KeyIter:
    def __iter__(self) -> KeyIter: ...
    def __next__(self) -> Key: ...

class GpgmeError(RuntimeError):
    def __init__(self, source: ErrSource, code: ErrCode, message: Optional[str] = None) -> None: ...
    source: ErrSource
    code: ErrCode
    strerror: str
    result: ImportResult | GenkeyResult

class DataEncoding(enum.IntEnum):
    NONE: int
    BINARY: int
    BASE64: int
    ARMOR: int

class PubkeyAlgo(enum.IntEnum):
    RSA: int
    RSA_E: int
    RSA_S: int
    ELG_E: int
    DSA: int
    ELG: int
    ECDSA: int
    ECDH: int
    EDDSA: int

class HashAlgo(enum.IntEnum):
    NONE: int
    MD5: int
    SHA1: int
    RMD160: int
    MD2: int
    TIGER: int
    HAVAL: int
    SHA256: int
    SHA384: int
    SHA512: int
    MD4: int
    CRC32: int
    CRC32_RFC1510: int
    CRC24_RFC2440: int

class SigMode(enum.IntEnum):
    NORMAL: int
    DETACH: int
    CLEAR: int

class Validity(enum.IntEnum):
    UNKNOWN: int
    UNDEFINED: int
    NEVER: int
    MARGINAL: int
    FULL: int
    ULTIMATE: int

class Protocol(enum.IntEnum):
    OpenPGP: int
    CMS: int
    GPGCONF: int
    ASSUAN: int
    G13: int
    UISERVER: int
    SPAWN: int
    DEFAULT: int
    UNKNOWN: int

class KeylistMode(enum.IntFlag):
    LOCAL: int
    EXTERN: int
    SIGS: int
    SIG_NOTATIONS: int
    WITH_SECRET: int
    WITH_TOFU: int
    WITH_KEYGRIP: int
    EPHEMERAL: int
    VALIDATE: int
    LOCATE: int
    FORCE_EXTERN: int
    LOCATE_EXTERNAL: int

class PinentryMode(enum.IntEnum):
    DEFAULT: int
    ASK: int
    CANCEL: int
    ERROR: int
    LOOPBACK: int

class ExportMode(enum.IntFlag):
    EXTERN: int
    MINIMAL: int
    SECRET: int
    RAW: int
    PKCS12: int
    SSH: int
    SECRET_SUBKEY: int

class SigNotationFlags(enum.IntFlag):
    HUMAN_READABLE: int
    CRITICAL: int

class Status(enum.IntEnum):
    EOF: int
    ENTER: int
    LEAVE: int
    ABORT: int
    GOODSIG: int
    BADSIG: int
    ERRSIG: int
    BADARMOR: int
    RSA_OR_IDEA: int
    KEYEXPIRED: int
    KEYREVOKED: int
    TRUST_UNDEFINED: int
    TRUST_NEVER: int
    TRUST_MARGINAL: int
    TRUST_FULLY: int
    TRUST_ULTIMATE: int
    SHM_INFO: int
    SHM_GET: int
    SHM_GET_BOOL: int
    SHM_GET_HIDDEN: int
    NEED_PASSPHRASE: int
    VALIDSIG: int
    SIG_ID: int
    ENC_TO: int
    NODATA: int
    BAD_PASSPHRASE: int
    NO_PUBKEY: int
    NO_SECKEY: int
    NEED_PASSPHRASE_SYM: int
    DECRYPTION_FAILED: int
    DECRYPTION_OKAY: int
    MISSING_PASSPHRASE: int
    GOOD_PASSPHRASE: int
    GOODMDC: int
    BADMDC: int
    ERRMDC: int
    IMPORTED: int
    IMPORT_OK: int
    IMPORT_PROBLEM: int
    IMPORT_RES: int
    FILE_START: int
    FILE_DONE: int
    FILE_ERROR: int
    BEGIN_DECRYPTION: int
    END_DECRYPTION: int
    BEGIN_ENCRYPTION: int
    END_ENCRYPTION: int
    DELETE_PROBLEM: int
    GET_BOOL: int
    GET_LINE: int
    GET_HIDDEN: int
    GOT_IT: int
    PROGRESS: int
    SIG_CREATED: int
    SESSION_KEY: int
    NOTATION_NAME: int
    NOTATION_DATA: int
    POLICY_URL: int
    BEGIN_STREAM: int
    END_STREAM: int
    KEY_CREATED: int
    USERID_HINT: int
    UNEXPECTED: int
    INV_RECP: int
    NO_RECP: int
    ALREADY_SIGNED: int
    SIGEXPIRED: int
    EXPSIG: int
    EXPKEYSIG: int
    TRUNCATED: int
    ERROR: int
    NEWSIG: int
    REVKEYSIG: int
    SIG_SUBPACKET: int
    NEED_PASSPHRASE_PIN: int
    SC_OP_FAILURE: int
    SC_OP_SUCCESS: int
    CARDCTRL: int
    BACKUP_KEY_CREATED: int
    PKA_TRUST_BAD: int
    PKA_TRUST_GOOD: int
    PLAINTEXT: int
    INV_SGNR: int
    NO_SGNR: int
    SUCCESS: int
    DECRYPTION_INFO: int
    PLAINTEXT_LENGTH: int
    MOUNTPOINT: int
    PINENTRY_LAUNCHED: int
    ATTRIBUTE: int
    BEGIN_SIGNING: int
    KEY_NOT_CREATED: int
    INQUIRE_MAXLEN: int
    FAILURE: int
    KEY_CONSIDERED: int
    TOFU_USER: int
    TOFU_STATS: int
    TOFU_STATS_LONG: int
    NOTATION_FLAGS: int
    DECRYPTION_COMPLIANCE_MODE: int
    VERIFICATION_COMPLIANCE_MODE: int
    CANCELED_BY_USER: int

class EncryptFlags(enum.IntFlag):
    ALWAYS_TRUST: int
    NO_ENCRYPT_TO: int
    PREPARE: int
    EXPECT_SIGN: int
    NO_COMPRESS: int
    SYMMETRIC: int
    THROW_KEYIDS: int
    WRAP: int
    WANT_ADDRESS: int

class Sigsum(enum.IntFlag):
    VALID: int
    GREEN: int
    RED: int
    KEY_REVOKED: int
    KEY_EXPIRED: int
    SIG_EXPIRED: int
    KEY_MISSING: int
    CRL_MISSING: int
    CRL_TOO_OLD: int
    BAD_POLICY: int
    SYS_ERROR: int
    TOFU_CONFLICT: int

class Import(enum.IntFlag):
    NEW: int
    UID: int
    SIG: int
    SUBKEY: int
    SECRET: int

class Delete(enum.IntFlag):
    ALLOW_SECRET: int
    FORCE: int

class ErrSource(enum.IntEnum):
    UNKNOWN: int
    GCRYPT: int
    GPG: int
    GPGSM: int
    GPGAGENT: int
    PINENTRY: int
    SCD: int
    GPGME: int
    KEYBOX: int
    KSBA: int
    DIRMNGR: int
    GSTI: int
    GPA: int
    KLEO: int
    G13: int
    ASSUAN: int
    TPM2D: int
    TLS: int
    TKD: int
    ANY: int
    USER_1: int
    USER_2: int
    USER_3: int
    USER_4: int

class ErrCode(enum.IntEnum):
    NO_ERROR: int
    GENERAL: int
    UNKNOWN_PACKET: int
    UNKNOWN_VERSION: int
    PUBKEY_ALGO: int
    DIGEST_ALGO: int
    BAD_PUBKEY: int
    BAD_SECKEY: int
    BAD_SIGNATURE: int
    NO_PUBKEY: int
    CHECKSUM: int
    BAD_PASSPHRASE: int
    CIPHER_ALGO: int
    KEYRING_OPEN: int
    INV_PACKET: int
    INV_ARMOR: int
    NO_USER_ID: int
    NO_SECKEY: int
    WRONG_SECKEY: int
    BAD_KEY: int
    COMPR_ALGO: int
    NO_PRIME: int
    NO_ENCODING_METHOD: int
    NO_ENCRYPTION_SCHEME: int
    NO_SIGNATURE_SCHEME: int
    INV_ATTR: int
    NO_VALUE: int
    NOT_FOUND: int
    VALUE_NOT_FOUND: int
    SYNTAX: int
    BAD_MPI: int
    INV_PASSPHRASE: int
    SIG_CLASS: int
    RESOURCE_LIMIT: int
    INV_KEYRING: int
    TRUSTDB: int
    BAD_CERT: int
    INV_USER_ID: int
    UNEXPECTED: int
    TIME_CONFLICT: int
    KEYSERVER: int
    WRONG_PUBKEY_ALGO: int
    TRIBUTE_TO_D_A: int
    WEAK_KEY: int
    INV_KEYLEN: int
    INV_ARG: int
    BAD_URI: int
    INV_URI: int
    NETWORK: int
    UNKNOWN_HOST: int
    SELFTEST_FAILED: int
    NOT_ENCRYPTED: int
    NOT_PROCESSED: int
    UNUSABLE_PUBKEY: int
    UNUSABLE_SECKEY: int
    INV_VALUE: int
    BAD_CERT_CHAIN: int
    MISSING_CERT: int
    NO_DATA: int
    BUG: int
    NOT_SUPPORTED: int
    INV_OP: int
    TIMEOUT: int
    INTERNAL: int
    EOF_GCRYPT: int
    INV_OBJ: int
    TOO_SHORT: int
    TOO_LARGE: int
    NO_OBJ: int
    NOT_IMPLEMENTED: int
    CONFLICT: int
    INV_CIPHER_MODE: int
    INV_FLAG: int
    INV_HANDLE: int
    TRUNCATED: int
    INCOMPLETE_LINE: int
    INV_RESPONSE: int
    NO_AGENT: int
    AGENT: int
    INV_DATA: int
    ASSUAN_SERVER_FAULT: int
    ASSUAN: int
    INV_SESSION_KEY: int
    INV_SEXP: int
    UNSUPPORTED_ALGORITHM: int
    NO_PIN_ENTRY: int
    PIN_ENTRY: int
    BAD_PIN: int
    INV_NAME: int
    BAD_DATA: int
    INV_PARAMETER: int
    WRONG_CARD: int
    NO_DIRMNGR: int
    DIRMNGR: int
    CERT_REVOKED: int
    NO_CRL_KNOWN: int
    CRL_TOO_OLD: int
    LINE_TOO_LONG: int
    NOT_TRUSTED: int
    CANCELED: int
    BAD_CA_CERT: int
    CERT_EXPIRED: int
    CERT_TOO_YOUNG: int
    UNSUPPORTED_CERT: int
    UNKNOWN_SEXP: int
    UNSUPPORTED_PROTECTION: int
    CORRUPTED_PROTECTION: int
    AMBIGUOUS_NAME: int
    CARD: int
    CARD_RESET: int
    CARD_REMOVED: int
    INV_CARD: int
    CARD_NOT_PRESENT: int
    NO_PKCS15_APP: int
    NOT_CONFIRMED: int
    CONFIGURATION: int
    NO_POLICY_MATCH: int
    INV_INDEX: int
    INV_ID: int
    NO_SCDAEMON: int
    SCDAEMON: int
    UNSUPPORTED_PROTOCOL: int
    BAD_PIN_METHOD: int
    CARD_NOT_INITIALIZED: int
    UNSUPPORTED_OPERATION: int
    WRONG_KEY_USAGE: int
    NOTHING_FOUND: int
    WRONG_BLOB_TYPE: int
    MISSING_VALUE: int
    HARDWARE: int
    PIN_BLOCKED: int
    USE_CONDITIONS: int
    PIN_NOT_SYNCED: int
    INV_CRL: int
    BAD_BER: int
    INV_BER: int
    ELEMENT_NOT_FOUND: int
    IDENTIFIER_NOT_FOUND: int
    INV_TAG: int
    INV_LENGTH: int
    INV_KEYINFO: int
    UNEXPECTED_TAG: int
    NOT_DER_ENCODED: int
    NO_CMS_OBJ: int
    INV_CMS_OBJ: int
    UNKNOWN_CMS_OBJ: int
    UNSUPPORTED_CMS_OBJ: int
    UNSUPPORTED_ENCODING: int
    UNSUPPORTED_CMS_VERSION: int
    UNKNOWN_ALGORITHM: int
    INV_ENGINE: int
    PUBKEY_NOT_TRUSTED: int
    DECRYPT_FAILED: int
    KEY_EXPIRED: int
    SIG_EXPIRED: int
    ENCODING_PROBLEM: int
    INV_STATE: int
    DUP_VALUE: int
    MISSING_ACTION: int
    MODULE_NOT_FOUND: int
    INV_OID_STRING: int
    INV_TIME: int
    INV_CRL_OBJ: int
    UNSUPPORTED_CRL_VERSION: int
    INV_CERT_OBJ: int
    UNKNOWN_NAME: int
    LOCALE_PROBLEM: int
    NOT_LOCKED: int
    PROTOCOL_VIOLATION: int
    INV_MAC: int
    INV_REQUEST: int
    UNKNOWN_EXTN: int
    UNKNOWN_CRIT_EXTN: int
    LOCKED: int
    UNKNOWN_OPTION: int
    UNKNOWN_COMMAND: int
    NOT_OPERATIONAL: int
    NO_PASSPHRASE: int
    NO_PIN: int
    NOT_ENABLED: int
    NO_ENGINE: int
    MISSING_KEY: int
    TOO_MANY: int
    LIMIT_REACHED: int
    NOT_INITIALIZED: int
    MISSING_ISSUER_CERT: int
    NO_KEYSERVER: int
    INV_CURVE: int
    UNKNOWN_CURVE: int
    DUP_KEY: int
    AMBIGUOUS: int
    NO_CRYPT_CTX: int
    WRONG_CRYPT_CTX: int
    BAD_CRYPT_CTX: int
    CRYPT_CTX_CONFLICT: int
    BROKEN_PUBKEY: int
    BROKEN_SECKEY: int
    MAC_ALGO: int
    FULLY_CANCELED: int
    UNFINISHED: int
    BUFFER_TOO_SHORT: int
    SEXP_INV_LEN_SPEC: int
    SEXP_STRING_TOO_LONG: int
    SEXP_UNMATCHED_PAREN: int
    SEXP_NOT_CANONICAL: int
    SEXP_BAD_CHARACTER: int
    SEXP_BAD_QUOTATION: int
    SEXP_ZERO_PREFIX: int
    SEXP_NESTED_DH: int
    SEXP_UNMATCHED_DH: int
    SEXP_UNEXPECTED_PUNC: int
    SEXP_BAD_HEX_CHAR: int
    SEXP_ODD_HEX_NUMBERS: int
    SEXP_BAD_OCT_CHAR: int
    SUBKEYS_EXP_OR_REV: int
    DB_CORRUPTED: int
    SERVER_FAILED: int
    NO_NAME: int
    NO_KEY: int
    LEGACY_KEY: int
    REQUEST_TOO_SHORT: int
    REQUEST_TOO_LONG: int
    OBJ_TERM_STATE: int
    NO_CERT_CHAIN: int
    CERT_TOO_LARGE: int
    INV_RECORD: int
    BAD_MAC: int
    UNEXPECTED_MSG: int
    COMPR_FAILED: int
    WOULD_WRAP: int
    FATAL_ALERT: int
    NO_CIPHER: int
    MISSING_CLIENT_CERT: int
    CLOSE_NOTIFY: int
    TICKET_EXPIRED: int
    BAD_TICKET: int
    UNKNOWN_IDENTITY: int
    BAD_HS_CERT: int
    BAD_HS_CERT_REQ: int
    BAD_HS_CERT_VER: int
    BAD_HS_CHANGE_CIPHER: int
    BAD_HS_CLIENT_HELLO: int
    BAD_HS_SERVER_HELLO: int
    BAD_HS_SERVER_HELLO_DONE: int
    BAD_HS_FINISHED: int
    BAD_HS_SERVER_KEX: int
    BAD_HS_CLIENT_KEX: int
    BOGUS_STRING: int
    FORBIDDEN: int
    KEY_DISABLED: int
    KEY_ON_CARD: int
    INV_LOCK_OBJ: int
    TRUE: int
    FALSE: int
    ASS_GENERAL: int
    ASS_ACCEPT_FAILED: int
    ASS_CONNECT_FAILED: int
    ASS_INV_RESPONSE: int
    ASS_INV_VALUE: int
    ASS_INCOMPLETE_LINE: int
    ASS_LINE_TOO_LONG: int
    ASS_NESTED_COMMANDS: int
    ASS_NO_DATA_CB: int
    ASS_NO_INQUIRE_CB: int
    ASS_NOT_A_SERVER: int
    ASS_NOT_A_CLIENT: int
    ASS_SERVER_START: int
    ASS_READ_ERROR: int
    ASS_WRITE_ERROR: int
    ASS_TOO_MUCH_DATA: int
    ASS_UNEXPECTED_CMD: int
    ASS_UNKNOWN_CMD: int
    ASS_SYNTAX: int
    ASS_CANCELED: int
    ASS_NO_INPUT: int
    ASS_NO_OUTPUT: int
    ASS_PARAMETER: int
    ASS_UNKNOWN_INQUIRE: int
    ENGINE_TOO_OLD: int
    WINDOW_TOO_SMALL: int
    WINDOW_TOO_LARGE: int
    MISSING_ENVVAR: int
    USER_ID_EXISTS: int
    NAME_EXISTS: int
    DUP_NAME: int
    TOO_YOUNG: int
    TOO_OLD: int
    UNKNOWN_FLAG: int
    INV_ORDER: int
    ALREADY_FETCHED: int
    TRY_LATER: int
    WRONG_NAME: int
    NO_AUTH: int
    BAD_AUTH: int
    NO_KEYBOXD: int
    KEYBOXD: int
    NO_SERVICE: int
    SERVICE: int
    BAD_PUK: int
    NO_RESET_CODE: int
    BAD_RESET_CODE: int
    SYSTEM_BUG: int
    DNS_UNKNOWN: int
    DNS_SECTION: int
    DNS_ADDRESS: int
    DNS_NO_QUERY: int
    DNS_NO_ANSWER: int
    DNS_CLOSED: int
    DNS_VERIFY: int
    DNS_TIMEOUT: int
    LDAP_GENERAL: int
    LDAP_ATTR_GENERAL: int
    LDAP_NAME_GENERAL: int
    LDAP_SECURITY_GENERAL: int
    LDAP_SERVICE_GENERAL: int
    LDAP_UPDATE_GENERAL: int
    LDAP_E_GENERAL: int
    LDAP_X_GENERAL: int
    LDAP_OTHER_GENERAL: int
    LDAP_X_CONNECTING: int
    LDAP_REFERRAL_LIMIT: int
    LDAP_CLIENT_LOOP: int
    LDAP_NO_RESULTS: int
    LDAP_CONTROL_NOT_FOUND: int
    LDAP_NOT_SUPPORTED: int
    LDAP_CONNECT: int
    LDAP_NO_MEMORY: int
    LDAP_PARAM: int
    LDAP_USER_CANCELLED: int
    LDAP_FILTER: int
    LDAP_AUTH_UNKNOWN: int
    LDAP_TIMEOUT: int
    LDAP_DECODING: int
    LDAP_ENCODING: int
    LDAP_LOCAL: int
    LDAP_SERVER_DOWN: int
    LDAP_SUCCESS: int
    LDAP_OPERATIONS: int
    LDAP_PROTOCOL: int
    LDAP_TIMELIMIT: int
    LDAP_SIZELIMIT: int
    LDAP_COMPARE_FALSE: int
    LDAP_COMPARE_TRUE: int
    LDAP_UNSUPPORTED_AUTH: int
    LDAP_STRONG_AUTH_RQRD: int
    LDAP_PARTIAL_RESULTS: int
    LDAP_REFERRAL: int
    LDAP_ADMINLIMIT: int
    LDAP_UNAVAIL_CRIT_EXTN: int
    LDAP_CONFIDENT_RQRD: int
    LDAP_SASL_BIND_INPROG: int
    LDAP_NO_SUCH_ATTRIBUTE: int
    LDAP_UNDEFINED_TYPE: int
    LDAP_BAD_MATCHING: int
    LDAP_CONST_VIOLATION: int
    LDAP_TYPE_VALUE_EXISTS: int
    LDAP_INV_SYNTAX: int
    LDAP_NO_SUCH_OBJ: int
    LDAP_ALIAS_PROBLEM: int
    LDAP_INV_DN_SYNTAX: int
    LDAP_IS_LEAF: int
    LDAP_ALIAS_DEREF: int
    LDAP_X_PROXY_AUTH_FAIL: int
    LDAP_BAD_AUTH: int
    LDAP_INV_CREDENTIALS: int
    LDAP_INSUFFICIENT_ACC: int
    LDAP_BUSY: int
    LDAP_UNAVAILABLE: int
    LDAP_UNWILL_TO_PERFORM: int
    LDAP_LOOP_DETECT: int
    LDAP_NAMING_VIOLATION: int
    LDAP_OBJ_CLS_VIOLATION: int
    LDAP_NOT_ALLOW_NONLEAF: int
    LDAP_NOT_ALLOW_ON_RDN: int
    LDAP_ALREADY_EXISTS: int
    LDAP_NO_OBJ_CLASS_MODS: int
    LDAP_RESULTS_TOO_LARGE: int
    LDAP_AFFECTS_MULT_DSAS: int
    LDAP_VLV: int
    LDAP_OTHER: int
    LDAP_CUP_RESOURCE_LIMIT: int
    LDAP_CUP_SEC_VIOLATION: int
    LDAP_CUP_INV_DATA: int
    LDAP_CUP_UNSUP_SCHEME: int
    LDAP_CUP_RELOAD: int
    LDAP_CANCELLED: int
    LDAP_NO_SUCH_OPERATION: int
    LDAP_TOO_LATE: int
    LDAP_CANNOT_CANCEL: int
    LDAP_ASSERTION_FAILED: int
    LDAP_PROX_AUTH_DENIED: int
    USER_1: int
    USER_2: int
    USER_3: int
    USER_4: int
    USER_5: int
    USER_6: int
    USER_7: int
    USER_8: int
    USER_9: int
    USER_10: int
    USER_11: int
    USER_12: int
    USER_13: int
    USER_14: int
    USER_15: int
    USER_16: int
    SQL_OK: int
    SQL_ERROR: int
    SQL_INTERNAL: int
    SQL_PERM: int
    SQL_ABORT: int
    SQL_BUSY: int
    SQL_LOCKED: int
    SQL_NOMEM: int
    SQL_READONLY: int
    SQL_INTERRUPT: int
    SQL_IOERR: int
    SQL_CORRUPT: int
    SQL_NOTFOUND: int
    SQL_FULL: int
    SQL_CANTOPEN: int
    SQL_PROTOCOL: int
    SQL_EMPTY: int
    SQL_SCHEMA: int
    SQL_TOOBIG: int
    SQL_CONSTRAINT: int
    SQL_MISMATCH: int
    SQL_MISUSE: int
    SQL_NOLFS: int
    SQL_AUTH: int
    SQL_FORMAT: int
    SQL_RANGE: int
    SQL_NOTADB: int
    SQL_NOTICE: int
    SQL_WARNING: int
    SQL_ROW: int
    SQL_DONE: int
    MISSING_ERRNO: int
    UNKNOWN_ERRNO: int
    EOF: int
    E2BIG: int
    EACCES: int
    EADDRINUSE: int
    EADDRNOTAVAIL: int
    EADV: int
    EAFNOSUPPORT: int
    EAGAIN: int
    EALREADY: int
    EAUTH: int
    EBACKGROUND: int
    EBADE: int
    EBADF: int
    EBADFD: int
    EBADMSG: int
    EBADR: int
    EBADRPC: int
    EBADRQC: int
    EBADSLT: int
    EBFONT: int
    EBUSY: int
    ECANCELED: int
    ECHILD: int
    ECHRNG: int
    ECOMM: int
    ECONNABORTED: int
    ECONNREFUSED: int
    ECONNRESET: int
    ED: int
    EDEADLK: int
    EDEADLOCK: int
    EDESTADDRREQ: int
    EDIED: int
    EDOM: int
    EDOTDOT: int
    EDQUOT: int
    EEXIST: int
    EFAULT: int
    EFBIG: int
    EFTYPE: int
    EGRATUITOUS: int
    EGREGIOUS: int
    EHOSTDOWN: int
    EHOSTUNREACH: int
    EIDRM: int
    EIEIO: int
    EILSEQ: int
    EINPROGRESS: int
    EINTR: int
    EINVAL: int
    EIO: int
    EISCONN: int
    EISDIR: int
    EISNAM: int
    EL2HLT: int
    EL2NSYNC: int
    EL3HLT: int
    EL3RST: int
    ELIBACC: int
    ELIBBAD: int
    ELIBEXEC: int
    ELIBMAX: int
    ELIBSCN: int
    ELNRNG: int
    ELOOP: int
    EMEDIUMTYPE: int
    EMFILE: int
    EMLINK: int
    EMSGSIZE: int
    EMULTIHOP: int
    ENAMETOOLONG: int
    ENAVAIL: int
    ENEEDAUTH: int
    ENETDOWN: int
    ENETRESET: int
    ENETUNREACH: int
    ENFILE: int
    ENOANO: int
    ENOBUFS: int
    ENOCSI: int
    ENODATA: int
    ENODEV: int
    ENOENT: int
    ENOEXEC: int
    ENOLCK: int
    ENOLINK: int
    ENOMEDIUM: int
    ENOMEM: int
    ENOMSG: int
    ENONET: int
    ENOPKG: int
    ENOPROTOOPT: int
    ENOSPC: int
    ENOSR: int
    ENOSTR: int
    ENOSYS: int
    ENOTBLK: int
    ENOTCONN: int
    ENOTDIR: int
    ENOTEMPTY: int
    ENOTNAM: int
    ENOTSOCK: int
    ENOTSUP: int
    ENOTTY: int
    ENOTUNIQ: int
    ENXIO: int
    EOPNOTSUPP: int
    EOVERFLOW: int
    EPERM: int
    EPFNOSUPPORT: int
    EPIPE: int
    EPROCLIM: int
    EPROCUNAVAIL: int
    EPROGMISMATCH: int
    EPROGUNAVAIL: int
    EPROTO: int
    EPROTONOSUPPORT: int
    EPROTOTYPE: int
    ERANGE: int
    EREMCHG: int
    EREMOTE: int
    EREMOTEIO: int
    ERESTART: int
    EROFS: int
    ERPCMISMATCH: int
    ESHUTDOWN: int
    ESOCKTNOSUPPORT: int
    ESPIPE: int
    ESRCH: int
    ESRMNT: int
    ESTALE: int
    ESTRPIPE: int
    ETIME: int
    ETIMEDOUT: int
    ETOOMANYREFS: int
    ETXTBSY: int
    EUCLEAN: int
    EUNATCH: int
    EUSERS: int
    EWOULDBLOCK: int
    EXDEV: int
    EXFULL: int

gpgme_version: str



# pygpgme - a Python wrapper for the gpgme library
# Copyright (C) 2006  James Henstridge
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
PyGPGME, a Python wrapper for the GPGME library.

PyGPGME is a Python module that lets you sign, verify, encrypt and decrypt
messages using the OpenPGP format.

It is built on top of the GNU Privacy Guard and the GPGME library.

All interaction begins by creating a Context.
"""

from gpgme._gpgme import *

__version__ = '0.6'

DATA_ENCODING_NONE = DataEncoding.NONE
DATA_ENCODING_BINARY = DataEncoding.BINARY
DATA_ENCODING_BASE64 = DataEncoding.BASE64
DATA_ENCODING_ARMOR = DataEncoding.ARMOR

PK_RSA = PubkeyAlgo.RSA
PK_RSA_E = PubkeyAlgo.RSA_E
PK_RSA_S = PubkeyAlgo.RSA_S
PK_ELG_E = PubkeyAlgo.ELG_E
PK_DSA = PubkeyAlgo.DSA
PK_ELG = PubkeyAlgo.ELG

MD_NONE = HashAlgo.NONE
MD_MD5 = HashAlgo.MD5
MD_SHA1 = HashAlgo.SHA1
MD_RMD160 = HashAlgo.RMD160
MD_MD2 = HashAlgo.MD2
MD_TIGER = HashAlgo.TIGER
MD_HAVAL = HashAlgo.HAVAL
MD_SHA256 = HashAlgo.SHA256
MD_SHA384 = HashAlgo.SHA384
MD_SHA512 = HashAlgo.SHA512
MD_MD4 = HashAlgo.MD4
MD_CRC32 = HashAlgo.CRC32
MD_CRC32_RFC1510 = HashAlgo.CRC32_RFC1510
MD_CRC24_RFC2440 = HashAlgo.CRC24_RFC2440

SIG_MODE_NORMAL = SigMode.NORMAL
SIG_MODE_DETACH = SigMode.DETACH
SIG_MODE_CLEAR = SigMode.CLEAR

VALIDITY_UNKNOWN = Validity.UNKNOWN
VALIDITY_UNDEFINED = Validity.UNDEFINED
VALIDITY_NEVER = Validity.NEVER
VALIDITY_MARGINAL = Validity.MARGINAL
VALIDITY_FULL = Validity.FULL
VALIDITY_ULTIMATE = Validity.ULTIMATE

PROTOCOL_OpenPGP = Protocol.OpenPGP
PROTOCOL_CMS = Protocol.CMS

KEYLIST_MODE_LOCAL = KeylistMode.LOCAL
KEYLIST_MODE_EXTERN = KeylistMode.EXTERN
KEYLIST_MODE_SIGS = KeylistMode.SIGS
KEYLIST_MODE_VALIDATE = KeylistMode.VALIDATE

STATUS_EOF = Status.EOF
STATUS_ENTER = Status.ENTER
STATUS_LEAVE = Status.LEAVE
STATUS_ABORT = Status.ABORT
STATUS_GOODSIG = Status.GOODSIG
STATUS_BADSIG = Status.BADSIG
STATUS_ERRSIG = Status.ERRSIG
STATUS_BADARMOR = Status.BADARMOR
STATUS_RSA_OR_IDEA = Status.RSA_OR_IDEA
STATUS_KEYEXPIRED = Status.KEYEXPIRED
STATUS_KEYREVOKED = Status.KEYREVOKED
STATUS_TRUST_UNDEFINED = Status.TRUST_UNDEFINED
STATUS_TRUST_NEVER = Status.TRUST_NEVER
STATUS_TRUST_MARGINAL = Status.TRUST_MARGINAL
STATUS_TRUST_FULLY = Status.TRUST_FULLY
STATUS_TRUST_ULTIMATE = Status.TRUST_ULTIMATE
STATUS_SHM_INFO = Status.SHM_INFO
STATUS_SHM_GET = Status.SHM_GET
STATUS_SHM_GET_BOOL = Status.SHM_GET_BOOL
STATUS_SHM_GET_HIDDEN = Status.SHM_GET_HIDDEN
STATUS_NEED_PASSPHRASE = Status.NEED_PASSPHRASE
STATUS_VALIDSIG = Status.VALIDSIG
STATUS_SIG_ID = Status.SIG_ID
STATUS_ENC_TO = Status.ENC_TO
STATUS_NODATA = Status.NODATA
STATUS_BAD_PASSPHRASE = Status.BAD_PASSPHRASE
STATUS_NO_PUBKEY = Status.NO_PUBKEY
STATUS_NO_SECKEY = Status.NO_SECKEY
STATUS_NEED_PASSPHRASE_SYM = Status.NEED_PASSPHRASE_SYM
STATUS_DECRYPTION_FAILED = Status.DECRYPTION_FAILED
STATUS_DECRYPTION_OKAY = Status.DECRYPTION_OKAY
STATUS_MISSING_PASSPHRASE = Status.MISSING_PASSPHRASE
STATUS_GOOD_PASSPHRASE = Status.GOOD_PASSPHRASE
STATUS_GOODMDC = Status.GOODMDC
STATUS_BADMDC = Status.BADMDC
STATUS_ERRMDC = Status.ERRMDC
STATUS_IMPORTED = Status.IMPORTED
STATUS_IMPORT_OK = Status.IMPORT_OK
STATUS_IMPORT_PROBLEM = Status.IMPORT_PROBLEM
STATUS_IMPORT_RES = Status.IMPORT_RES
STATUS_FILE_START = Status.FILE_START
STATUS_FILE_DONE = Status.FILE_DONE
STATUS_FILE_ERROR = Status.FILE_ERROR
STATUS_BEGIN_DECRYPTION = Status.BEGIN_DECRYPTION
STATUS_END_DECRYPTION = Status.END_DECRYPTION
STATUS_BEGIN_ENCRYPTION = Status.BEGIN_ENCRYPTION
STATUS_END_ENCRYPTION = Status.END_ENCRYPTION
STATUS_DELETE_PROBLEM = Status.DELETE_PROBLEM
STATUS_GET_BOOL = Status.GET_BOOL
STATUS_GET_LINE = Status.GET_LINE
STATUS_GET_HIDDEN = Status.GET_HIDDEN
STATUS_GOT_IT = Status.GOT_IT
STATUS_PROGRESS = Status.PROGRESS
STATUS_SIG_CREATED = Status.SIG_CREATED
STATUS_SESSION_KEY = Status.SESSION_KEY
STATUS_NOTATION_NAME = Status.NOTATION_NAME
STATUS_NOTATION_DATA = Status.NOTATION_DATA
STATUS_POLICY_URL = Status.POLICY_URL
STATUS_BEGIN_STREAM = Status.BEGIN_STREAM
STATUS_END_STREAM = Status.END_STREAM
STATUS_KEY_CREATED = Status.KEY_CREATED
STATUS_USERID_HINT = Status.USERID_HINT
STATUS_UNEXPECTED = Status.UNEXPECTED
STATUS_INV_RECP = Status.INV_RECP
STATUS_NO_RECP = Status.NO_RECP
STATUS_ALREADY_SIGNED = Status.ALREADY_SIGNED
STATUS_SIGEXPIRED = Status.SIGEXPIRED
STATUS_EXPSIG = Status.EXPSIG
STATUS_EXPKEYSIG = Status.EXPKEYSIG
STATUS_TRUNCATED = Status.TRUNCATED
STATUS_ERROR = Status.ERROR
STATUS_NEWSIG = Status.NEWSIG
STATUS_REVKEYSIG = Status.REVKEYSIG

ENCRYPT_ALWAYS_TRUST = EncryptFlags.ALWAYS_TRUST

SIGSUM_VALID = Sigsum.VALID
SIGSUM_GREEN = Sigsum.GREEN
SIGSUM_RED = Sigsum.RED
SIGSUM_KEY_REVOKED = Sigsum.KEY_REVOKED
SIGSUM_KEY_EXPIRED = Sigsum.KEY_EXPIRED
SIGSUM_SIG_EXPIRED = Sigsum.SIG_EXPIRED
SIGSUM_KEY_MISSING = Sigsum.KEY_MISSING
SIGSUM_CRL_MISSING = Sigsum.CRL_MISSING
SIGSUM_CRL_TOO_OLD = Sigsum.CRL_TOO_OLD
SIGSUM_BAD_POLICY = Sigsum.BAD_POLICY
SIGSUM_SYS_ERROR = Sigsum.SYS_ERROR

IMPORT_NEW = Import.NEW
IMPORT_UID = Import.UID
IMPORT_SIG = Import.SIG
IMPORT_SUBKEY = Import.SUBKEY
IMPORT_SECRET = Import.SECRET

ERR_SOURCE_UNKNOWN = ErrSource.UNKNOWN
ERR_SOURCE_GCRYPT = ErrSource.GCRYPT
ERR_SOURCE_GPG = ErrSource.GPG
ERR_SOURCE_GPGSM = ErrSource.GPGSM
ERR_SOURCE_GPGAGENT = ErrSource.GPGAGENT
ERR_SOURCE_PINENTRY = ErrSource.PINENTRY
ERR_SOURCE_SCD = ErrSource.SCD
ERR_SOURCE_GPGME = ErrSource.GPGME
ERR_SOURCE_KEYBOX = ErrSource.KEYBOX
ERR_SOURCE_KSBA = ErrSource.KSBA
ERR_SOURCE_DIRMNGR = ErrSource.DIRMNGR
ERR_SOURCE_GSTI = ErrSource.GSTI
ERR_SOURCE_USER_1 = ErrSource.USER_1
ERR_SOURCE_USER_2 = ErrSource.USER_2
ERR_SOURCE_USER_3 = ErrSource.USER_3
ERR_SOURCE_USER_4 = ErrSource.USER_4

ERR_NO_ERROR = ErrCode.NO_ERROR
ERR_GENERAL = ErrCode.GENERAL
ERR_UNKNOWN_PACKET = ErrCode.UNKNOWN_PACKET
ERR_UNKNOWN_VERSION = ErrCode.UNKNOWN_VERSION
ERR_PUBKEY_ALGO = ErrCode.PUBKEY_ALGO
ERR_DIGEST_ALGO = ErrCode.DIGEST_ALGO
ERR_BAD_PUBKEY = ErrCode.BAD_PUBKEY
ERR_BAD_SECKEY = ErrCode.BAD_SECKEY
ERR_BAD_SIGNATURE = ErrCode.BAD_SIGNATURE
ERR_NO_PUBKEY = ErrCode.NO_PUBKEY
ERR_CHECKSUM = ErrCode.CHECKSUM
ERR_BAD_PASSPHRASE = ErrCode.BAD_PASSPHRASE
ERR_CIPHER_ALGO = ErrCode.CIPHER_ALGO
ERR_KEYRING_OPEN = ErrCode.KEYRING_OPEN
ERR_INV_PACKET = ErrCode.INV_PACKET
ERR_INV_ARMOR = ErrCode.INV_ARMOR
ERR_NO_USER_ID = ErrCode.NO_USER_ID
ERR_NO_SECKEY = ErrCode.NO_SECKEY
ERR_WRONG_SECKEY = ErrCode.WRONG_SECKEY
ERR_BAD_KEY = ErrCode.BAD_KEY
ERR_COMPR_ALGO = ErrCode.COMPR_ALGO
ERR_NO_PRIME = ErrCode.NO_PRIME
ERR_NO_ENCODING_METHOD = ErrCode.NO_ENCODING_METHOD
ERR_NO_ENCRYPTION_SCHEME = ErrCode.NO_ENCRYPTION_SCHEME
ERR_NO_SIGNATURE_SCHEME = ErrCode.NO_SIGNATURE_SCHEME
ERR_INV_ATTR = ErrCode.INV_ATTR
ERR_NO_VALUE = ErrCode.NO_VALUE
ERR_NOT_FOUND = ErrCode.NOT_FOUND
ERR_VALUE_NOT_FOUND = ErrCode.VALUE_NOT_FOUND
ERR_SYNTAX = ErrCode.SYNTAX
ERR_BAD_MPI = ErrCode.BAD_MPI
ERR_INV_PASSPHRASE = ErrCode.INV_PASSPHRASE
ERR_SIG_CLASS = ErrCode.SIG_CLASS
ERR_RESOURCE_LIMIT = ErrCode.RESOURCE_LIMIT
ERR_INV_KEYRING = ErrCode.INV_KEYRING
ERR_TRUSTDB = ErrCode.TRUSTDB
ERR_BAD_CERT = ErrCode.BAD_CERT
ERR_INV_USER_ID = ErrCode.INV_USER_ID
ERR_UNEXPECTED = ErrCode.UNEXPECTED
ERR_TIME_CONFLICT = ErrCode.TIME_CONFLICT
ERR_KEYSERVER = ErrCode.KEYSERVER
ERR_WRONG_PUBKEY_ALGO = ErrCode.WRONG_PUBKEY_ALGO
ERR_TRIBUTE_TO_D_A = ErrCode.TRIBUTE_TO_D_A
ERR_WEAK_KEY = ErrCode.WEAK_KEY
ERR_INV_KEYLEN = ErrCode.INV_KEYLEN
ERR_INV_ARG = ErrCode.INV_ARG
ERR_BAD_URI = ErrCode.BAD_URI
ERR_INV_URI = ErrCode.INV_URI
ERR_NETWORK = ErrCode.NETWORK
ERR_UNKNOWN_HOST = ErrCode.UNKNOWN_HOST
ERR_SELFTEST_FAILED = ErrCode.SELFTEST_FAILED
ERR_NOT_ENCRYPTED = ErrCode.NOT_ENCRYPTED
ERR_NOT_PROCESSED = ErrCode.NOT_PROCESSED
ERR_UNUSABLE_PUBKEY = ErrCode.UNUSABLE_PUBKEY
ERR_UNUSABLE_SECKEY = ErrCode.UNUSABLE_SECKEY
ERR_INV_VALUE = ErrCode.INV_VALUE
ERR_BAD_CERT_CHAIN = ErrCode.BAD_CERT_CHAIN
ERR_MISSING_CERT = ErrCode.MISSING_CERT
ERR_NO_DATA = ErrCode.NO_DATA
ERR_BUG = ErrCode.BUG
ERR_NOT_SUPPORTED = ErrCode.NOT_SUPPORTED
ERR_INV_OP = ErrCode.INV_OP
ERR_TIMEOUT = ErrCode.TIMEOUT
ERR_INTERNAL = ErrCode.INTERNAL
ERR_EOF_GCRYPT = ErrCode.EOF_GCRYPT
ERR_INV_OBJ = ErrCode.INV_OBJ
ERR_TOO_SHORT = ErrCode.TOO_SHORT
ERR_TOO_LARGE = ErrCode.TOO_LARGE
ERR_NO_OBJ = ErrCode.NO_OBJ
ERR_NOT_IMPLEMENTED = ErrCode.NOT_IMPLEMENTED
ERR_CONFLICT = ErrCode.CONFLICT
ERR_INV_CIPHER_MODE = ErrCode.INV_CIPHER_MODE
ERR_INV_FLAG = ErrCode.INV_FLAG
ERR_INV_HANDLE = ErrCode.INV_HANDLE
ERR_TRUNCATED = ErrCode.TRUNCATED
ERR_INCOMPLETE_LINE = ErrCode.INCOMPLETE_LINE
ERR_INV_RESPONSE = ErrCode.INV_RESPONSE
ERR_NO_AGENT = ErrCode.NO_AGENT
ERR_AGENT = ErrCode.AGENT
ERR_INV_DATA = ErrCode.INV_DATA
ERR_ASSUAN_SERVER_FAULT = ErrCode.ASSUAN_SERVER_FAULT
ERR_ASSUAN = ErrCode.ASSUAN
ERR_INV_SESSION_KEY = ErrCode.INV_SESSION_KEY
ERR_INV_SEXP = ErrCode.INV_SEXP
ERR_UNSUPPORTED_ALGORITHM = ErrCode.UNSUPPORTED_ALGORITHM
ERR_NO_PIN_ENTRY = ErrCode.NO_PIN_ENTRY
ERR_PIN_ENTRY = ErrCode.PIN_ENTRY
ERR_BAD_PIN = ErrCode.BAD_PIN
ERR_INV_NAME = ErrCode.INV_NAME
ERR_BAD_DATA = ErrCode.BAD_DATA
ERR_INV_PARAMETER = ErrCode.INV_PARAMETER
ERR_WRONG_CARD = ErrCode.WRONG_CARD
ERR_NO_DIRMNGR = ErrCode.NO_DIRMNGR
ERR_DIRMNGR = ErrCode.DIRMNGR
ERR_CERT_REVOKED = ErrCode.CERT_REVOKED
ERR_NO_CRL_KNOWN = ErrCode.NO_CRL_KNOWN
ERR_CRL_TOO_OLD = ErrCode.CRL_TOO_OLD
ERR_LINE_TOO_LONG = ErrCode.LINE_TOO_LONG
ERR_NOT_TRUSTED = ErrCode.NOT_TRUSTED
ERR_CANCELED = ErrCode.CANCELED
ERR_BAD_CA_CERT = ErrCode.BAD_CA_CERT
ERR_CERT_EXPIRED = ErrCode.CERT_EXPIRED
ERR_CERT_TOO_YOUNG = ErrCode.CERT_TOO_YOUNG
ERR_UNSUPPORTED_CERT = ErrCode.UNSUPPORTED_CERT
ERR_UNKNOWN_SEXP = ErrCode.UNKNOWN_SEXP
ERR_UNSUPPORTED_PROTECTION = ErrCode.UNSUPPORTED_PROTECTION
ERR_CORRUPTED_PROTECTION = ErrCode.CORRUPTED_PROTECTION
ERR_AMBIGUOUS_NAME = ErrCode.AMBIGUOUS_NAME
ERR_CARD = ErrCode.CARD
ERR_CARD_RESET = ErrCode.CARD_RESET
ERR_CARD_REMOVED = ErrCode.CARD_REMOVED
ERR_INV_CARD = ErrCode.INV_CARD
ERR_CARD_NOT_PRESENT = ErrCode.CARD_NOT_PRESENT
ERR_NO_PKCS15_APP = ErrCode.NO_PKCS15_APP
ERR_NOT_CONFIRMED = ErrCode.NOT_CONFIRMED
ERR_CONFIGURATION = ErrCode.CONFIGURATION
ERR_NO_POLICY_MATCH = ErrCode.NO_POLICY_MATCH
ERR_INV_INDEX = ErrCode.INV_INDEX
ERR_INV_ID = ErrCode.INV_ID
ERR_NO_SCDAEMON = ErrCode.NO_SCDAEMON
ERR_SCDAEMON = ErrCode.SCDAEMON
ERR_UNSUPPORTED_PROTOCOL = ErrCode.UNSUPPORTED_PROTOCOL
ERR_BAD_PIN_METHOD = ErrCode.BAD_PIN_METHOD
ERR_CARD_NOT_INITIALIZED = ErrCode.CARD_NOT_INITIALIZED
ERR_UNSUPPORTED_OPERATION = ErrCode.UNSUPPORTED_OPERATION
ERR_WRONG_KEY_USAGE = ErrCode.WRONG_KEY_USAGE
ERR_NOTHING_FOUND = ErrCode.NOTHING_FOUND
ERR_WRONG_BLOB_TYPE = ErrCode.WRONG_BLOB_TYPE
ERR_MISSING_VALUE = ErrCode.MISSING_VALUE
ERR_HARDWARE = ErrCode.HARDWARE
ERR_PIN_BLOCKED = ErrCode.PIN_BLOCKED
ERR_USE_CONDITIONS = ErrCode.USE_CONDITIONS
ERR_PIN_NOT_SYNCED = ErrCode.PIN_NOT_SYNCED
ERR_INV_CRL = ErrCode.INV_CRL
ERR_BAD_BER = ErrCode.BAD_BER
ERR_INV_BER = ErrCode.INV_BER
ERR_ELEMENT_NOT_FOUND = ErrCode.ELEMENT_NOT_FOUND
ERR_IDENTIFIER_NOT_FOUND = ErrCode.IDENTIFIER_NOT_FOUND
ERR_INV_TAG = ErrCode.INV_TAG
ERR_INV_LENGTH = ErrCode.INV_LENGTH
ERR_INV_KEYINFO = ErrCode.INV_KEYINFO
ERR_UNEXPECTED_TAG = ErrCode.UNEXPECTED_TAG
ERR_NOT_DER_ENCODED = ErrCode.NOT_DER_ENCODED
ERR_NO_CMS_OBJ = ErrCode.NO_CMS_OBJ
ERR_INV_CMS_OBJ = ErrCode.INV_CMS_OBJ
ERR_UNKNOWN_CMS_OBJ = ErrCode.UNKNOWN_CMS_OBJ
ERR_UNSUPPORTED_CMS_OBJ = ErrCode.UNSUPPORTED_CMS_OBJ
ERR_UNSUPPORTED_ENCODING = ErrCode.UNSUPPORTED_ENCODING
ERR_UNSUPPORTED_CMS_VERSION = ErrCode.UNSUPPORTED_CMS_VERSION
ERR_UNKNOWN_ALGORITHM = ErrCode.UNKNOWN_ALGORITHM
ERR_INV_ENGINE = ErrCode.INV_ENGINE
ERR_PUBKEY_NOT_TRUSTED = ErrCode.PUBKEY_NOT_TRUSTED
ERR_DECRYPT_FAILED = ErrCode.DECRYPT_FAILED
ERR_KEY_EXPIRED = ErrCode.KEY_EXPIRED
ERR_SIG_EXPIRED = ErrCode.SIG_EXPIRED
ERR_ENCODING_PROBLEM = ErrCode.ENCODING_PROBLEM
ERR_INV_STATE = ErrCode.INV_STATE
ERR_DUP_VALUE = ErrCode.DUP_VALUE
ERR_MISSING_ACTION = ErrCode.MISSING_ACTION
ERR_MODULE_NOT_FOUND = ErrCode.MODULE_NOT_FOUND
ERR_INV_OID_STRING = ErrCode.INV_OID_STRING
ERR_INV_TIME = ErrCode.INV_TIME
ERR_INV_CRL_OBJ = ErrCode.INV_CRL_OBJ
ERR_UNSUPPORTED_CRL_VERSION = ErrCode.UNSUPPORTED_CRL_VERSION
ERR_INV_CERT_OBJ = ErrCode.INV_CERT_OBJ
ERR_UNKNOWN_NAME = ErrCode.UNKNOWN_NAME
ERR_LOCALE_PROBLEM = ErrCode.LOCALE_PROBLEM
ERR_NOT_LOCKED = ErrCode.NOT_LOCKED
ERR_PROTOCOL_VIOLATION = ErrCode.PROTOCOL_VIOLATION
ERR_INV_MAC = ErrCode.INV_MAC
ERR_INV_REQUEST = ErrCode.INV_REQUEST
ERR_BUFFER_TOO_SHORT = ErrCode.BUFFER_TOO_SHORT
ERR_SEXP_INV_LEN_SPEC = ErrCode.SEXP_INV_LEN_SPEC
ERR_SEXP_STRING_TOO_LONG = ErrCode.SEXP_STRING_TOO_LONG
ERR_SEXP_UNMATCHED_PAREN = ErrCode.SEXP_UNMATCHED_PAREN
ERR_SEXP_NOT_CANONICAL = ErrCode.SEXP_NOT_CANONICAL
ERR_SEXP_BAD_CHARACTER = ErrCode.SEXP_BAD_CHARACTER
ERR_SEXP_BAD_QUOTATION = ErrCode.SEXP_BAD_QUOTATION
ERR_SEXP_ZERO_PREFIX = ErrCode.SEXP_ZERO_PREFIX
ERR_SEXP_NESTED_DH = ErrCode.SEXP_NESTED_DH
ERR_SEXP_UNMATCHED_DH = ErrCode.SEXP_UNMATCHED_DH
ERR_SEXP_UNEXPECTED_PUNC = ErrCode.SEXP_UNEXPECTED_PUNC
ERR_SEXP_BAD_HEX_CHAR = ErrCode.SEXP_BAD_HEX_CHAR
ERR_SEXP_ODD_HEX_NUMBERS = ErrCode.SEXP_ODD_HEX_NUMBERS
ERR_SEXP_BAD_OCT_CHAR = ErrCode.SEXP_BAD_OCT_CHAR
ERR_USER_1 = ErrCode.USER_1
ERR_USER_2 = ErrCode.USER_2
ERR_USER_3 = ErrCode.USER_3
ERR_USER_4 = ErrCode.USER_4
ERR_USER_5 = ErrCode.USER_5
ERR_USER_6 = ErrCode.USER_6
ERR_USER_7 = ErrCode.USER_7
ERR_USER_8 = ErrCode.USER_8
ERR_USER_9 = ErrCode.USER_9
ERR_USER_10 = ErrCode.USER_10
ERR_USER_11 = ErrCode.USER_11
ERR_USER_12 = ErrCode.USER_12
ERR_USER_13 = ErrCode.USER_13
ERR_USER_14 = ErrCode.USER_14
ERR_USER_15 = ErrCode.USER_15
ERR_USER_16 = ErrCode.USER_16
ERR_UNKNOWN_ERRNO = ErrCode.UNKNOWN_ERRNO
ERR_EOF = ErrCode.EOF
ERR_E2BIG = ErrCode.E2BIG
ERR_EACCES = ErrCode.EACCES
ERR_EADDRINUSE = ErrCode.EADDRINUSE
ERR_EADDRNOTAVAIL = ErrCode.EADDRNOTAVAIL
ERR_EADV = ErrCode.EADV
ERR_EAFNOSUPPORT = ErrCode.EAFNOSUPPORT
ERR_EAGAIN = ErrCode.EAGAIN
ERR_EALREADY = ErrCode.EALREADY
ERR_EAUTH = ErrCode.EAUTH
ERR_EBACKGROUND = ErrCode.EBACKGROUND
ERR_EBADE = ErrCode.EBADE
ERR_EBADF = ErrCode.EBADF
ERR_EBADFD = ErrCode.EBADFD
ERR_EBADMSG = ErrCode.EBADMSG
ERR_EBADR = ErrCode.EBADR
ERR_EBADRPC = ErrCode.EBADRPC
ERR_EBADRQC = ErrCode.EBADRQC
ERR_EBADSLT = ErrCode.EBADSLT
ERR_EBFONT = ErrCode.EBFONT
ERR_EBUSY = ErrCode.EBUSY
ERR_ECANCELED = ErrCode.ECANCELED
ERR_ECHILD = ErrCode.ECHILD
ERR_ECHRNG = ErrCode.ECHRNG
ERR_ECOMM = ErrCode.ECOMM
ERR_ECONNABORTED = ErrCode.ECONNABORTED
ERR_ECONNREFUSED = ErrCode.ECONNREFUSED
ERR_ECONNRESET = ErrCode.ECONNRESET
ERR_ED = ErrCode.ED
ERR_EDEADLK = ErrCode.EDEADLK
ERR_EDEADLOCK = ErrCode.EDEADLOCK
ERR_EDESTADDRREQ = ErrCode.EDESTADDRREQ
ERR_EDIED = ErrCode.EDIED
ERR_EDOM = ErrCode.EDOM
ERR_EDOTDOT = ErrCode.EDOTDOT
ERR_EDQUOT = ErrCode.EDQUOT
ERR_EEXIST = ErrCode.EEXIST
ERR_EFAULT = ErrCode.EFAULT
ERR_EFBIG = ErrCode.EFBIG
ERR_EFTYPE = ErrCode.EFTYPE
ERR_EGRATUITOUS = ErrCode.EGRATUITOUS
ERR_EGREGIOUS = ErrCode.EGREGIOUS
ERR_EHOSTDOWN = ErrCode.EHOSTDOWN
ERR_EHOSTUNREACH = ErrCode.EHOSTUNREACH
ERR_EIDRM = ErrCode.EIDRM
ERR_EIEIO = ErrCode.EIEIO
ERR_EILSEQ = ErrCode.EILSEQ
ERR_EINPROGRESS = ErrCode.EINPROGRESS
ERR_EINTR = ErrCode.EINTR
ERR_EINVAL = ErrCode.EINVAL
ERR_EIO = ErrCode.EIO
ERR_EISCONN = ErrCode.EISCONN
ERR_EISDIR = ErrCode.EISDIR
ERR_EISNAM = ErrCode.EISNAM
ERR_EL2HLT = ErrCode.EL2HLT
ERR_EL2NSYNC = ErrCode.EL2NSYNC
ERR_EL3HLT = ErrCode.EL3HLT
ERR_EL3RST = ErrCode.EL3RST
ERR_ELIBACC = ErrCode.ELIBACC
ERR_ELIBBAD = ErrCode.ELIBBAD
ERR_ELIBEXEC = ErrCode.ELIBEXEC
ERR_ELIBMAX = ErrCode.ELIBMAX
ERR_ELIBSCN = ErrCode.ELIBSCN
ERR_ELNRNG = ErrCode.ELNRNG
ERR_ELOOP = ErrCode.ELOOP
ERR_EMEDIUMTYPE = ErrCode.EMEDIUMTYPE
ERR_EMFILE = ErrCode.EMFILE
ERR_EMLINK = ErrCode.EMLINK
ERR_EMSGSIZE = ErrCode.EMSGSIZE
ERR_EMULTIHOP = ErrCode.EMULTIHOP
ERR_ENAMETOOLONG = ErrCode.ENAMETOOLONG
ERR_ENAVAIL = ErrCode.ENAVAIL
ERR_ENEEDAUTH = ErrCode.ENEEDAUTH
ERR_ENETDOWN = ErrCode.ENETDOWN
ERR_ENETRESET = ErrCode.ENETRESET
ERR_ENETUNREACH = ErrCode.ENETUNREACH
ERR_ENFILE = ErrCode.ENFILE
ERR_ENOANO = ErrCode.ENOANO
ERR_ENOBUFS = ErrCode.ENOBUFS
ERR_ENOCSI = ErrCode.ENOCSI
ERR_ENODATA = ErrCode.ENODATA
ERR_ENODEV = ErrCode.ENODEV
ERR_ENOENT = ErrCode.ENOENT
ERR_ENOEXEC = ErrCode.ENOEXEC
ERR_ENOLCK = ErrCode.ENOLCK
ERR_ENOLINK = ErrCode.ENOLINK
ERR_ENOMEDIUM = ErrCode.ENOMEDIUM
ERR_ENOMEM = ErrCode.ENOMEM
ERR_ENOMSG = ErrCode.ENOMSG
ERR_ENONET = ErrCode.ENONET
ERR_ENOPKG = ErrCode.ENOPKG
ERR_ENOPROTOOPT = ErrCode.ENOPROTOOPT
ERR_ENOSPC = ErrCode.ENOSPC
ERR_ENOSR = ErrCode.ENOSR
ERR_ENOSTR = ErrCode.ENOSTR
ERR_ENOSYS = ErrCode.ENOSYS
ERR_ENOTBLK = ErrCode.ENOTBLK
ERR_ENOTCONN = ErrCode.ENOTCONN
ERR_ENOTDIR = ErrCode.ENOTDIR
ERR_ENOTEMPTY = ErrCode.ENOTEMPTY
ERR_ENOTNAM = ErrCode.ENOTNAM
ERR_ENOTSOCK = ErrCode.ENOTSOCK
ERR_ENOTSUP = ErrCode.ENOTSUP
ERR_ENOTTY = ErrCode.ENOTTY
ERR_ENOTUNIQ = ErrCode.ENOTUNIQ
ERR_ENXIO = ErrCode.ENXIO
ERR_EOPNOTSUPP = ErrCode.EOPNOTSUPP
ERR_EOVERFLOW = ErrCode.EOVERFLOW
ERR_EPERM = ErrCode.EPERM
ERR_EPFNOSUPPORT = ErrCode.EPFNOSUPPORT
ERR_EPIPE = ErrCode.EPIPE
ERR_EPROCLIM = ErrCode.EPROCLIM
ERR_EPROCUNAVAIL = ErrCode.EPROCUNAVAIL
ERR_EPROGMISMATCH = ErrCode.EPROGMISMATCH
ERR_EPROGUNAVAIL = ErrCode.EPROGUNAVAIL
ERR_EPROTO = ErrCode.EPROTO
ERR_EPROTONOSUPPORT = ErrCode.EPROTONOSUPPORT
ERR_EPROTOTYPE = ErrCode.EPROTOTYPE
ERR_ERANGE = ErrCode.ERANGE
ERR_EREMCHG = ErrCode.EREMCHG
ERR_EREMOTE = ErrCode.EREMOTE
ERR_EREMOTEIO = ErrCode.EREMOTEIO
ERR_ERESTART = ErrCode.ERESTART
ERR_EROFS = ErrCode.EROFS
ERR_ERPCMISMATCH = ErrCode.ERPCMISMATCH
ERR_ESHUTDOWN = ErrCode.ESHUTDOWN
ERR_ESOCKTNOSUPPORT = ErrCode.ESOCKTNOSUPPORT
ERR_ESPIPE = ErrCode.ESPIPE
ERR_ESRCH = ErrCode.ESRCH
ERR_ESRMNT = ErrCode.ESRMNT
ERR_ESTALE = ErrCode.ESTALE
ERR_ESTRPIPE = ErrCode.ESTRPIPE
ERR_ETIME = ErrCode.ETIME
ERR_ETIMEDOUT = ErrCode.ETIMEDOUT
ERR_ETOOMANYREFS = ErrCode.ETOOMANYREFS
ERR_ETXTBSY = ErrCode.ETXTBSY
ERR_EUCLEAN = ErrCode.EUCLEAN
ERR_EUNATCH = ErrCode.EUNATCH
ERR_EUSERS = ErrCode.EUSERS
ERR_EWOULDBLOCK = ErrCode.EWOULDBLOCK
ERR_EXDEV = ErrCode.EXDEV
ERR_EXFULL = ErrCode.EXFULL



import gpgme
from io import BytesIO

ctx = gpgme.Context()
ctx.armor = True

key = ctx.get_key('B10A449E4CFB9A60A2DB996701AF93D991CFA34D')

plain = BytesIO(b'Hello World\n')
cipher = BytesIO()

ctx.encrypt([key], 0, plain, cipher)

print(cipher.getvalue())



import shlex
import subprocess

from setuptools import setup, Extension

def pkgconfig(*args):
    argv = ["pkg-config"] + list(args)
    output = subprocess.check_output(argv).decode('ASCII')
    return shlex.split(output)

gpgme_cflags = pkgconfig("--cflags", "gpgme >= 1.13.0")
gpgme_libs = pkgconfig("--libs", "gpgme >= 1.13.0")

setup(ext_modules=[
    Extension(
        'gpgme._gpgme',
        ['lib/gpgme.c',
         'lib/pygpgme-error.c',
         'lib/pygpgme-data.c',
         'lib/pygpgme-context.c',
         'lib/pygpgme-engine-info.c',
         'lib/pygpgme-key.c',
         'lib/pygpgme-signature.c',
         'lib/pygpgme-import.c',
         'lib/pygpgme-keyiter.c',
         'lib/pygpgme-constants.c',
         'lib/pygpgme-genkey.c',
         ],
        extra_compile_args=gpgme_cflags,
        extra_link_args=gpgme_libs)
])



# pygpgme - a Python wrapper for the gpgme library
# Copyright (C) 2006  James Henstridge
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from io import BytesIO
from textwrap import dedent
import unittest

import gpgme
from tests.util import GpgHomeTestCase

class ExportTestCase(GpgHomeTestCase):

    import_keys = ['signonly.pub', 'signonly.sec']

    def test_export_by_fingerprint(self) -> None:
        ctx = gpgme.Context()
        ctx.armor = True
        keydata = BytesIO()
        ctx.export('15E7CE9BF1771A4ABC550B31F540A569CB935A42', keydata)

        self.assertTrue(keydata.getvalue().startswith(
            b'-----BEGIN PGP PUBLIC KEY BLOCK-----\n'))

    def test_export_by_email(self) -> None:
        ctx = gpgme.Context()
        ctx.armor = True
        keydata = BytesIO()
        ctx.export('signonly@example.org', keydata)

        self.assertTrue(keydata.getvalue().startswith(
            b'-----BEGIN PGP PUBLIC KEY BLOCK-----\n'))

    def test_export_by_name(self) -> None:
        ctx = gpgme.Context()
        ctx.armor = True
        keydata = BytesIO()
        ctx.export('Sign Only', keydata)

        self.assertTrue(keydata.getvalue().startswith(
            b'-----BEGIN PGP PUBLIC KEY BLOCK-----\n'))

    def test_export_keys(self) -> None:
        ctx = gpgme.Context()
        ctx.armor = True
        key = ctx.get_key('15E7CE9BF1771A4ABC550B31F540A569CB935A42')
        keydata = BytesIO()
        ctx.export_keys([key], keydata)
        self.assertTrue(keydata.getvalue().startswith(
            b'-----BEGIN PGP PUBLIC KEY BLOCK-----\n'))
